/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.EventObject;


import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.draw2d.LightweightSystem;
import org.eclipse.draw2d.Viewport;
import org.eclipse.draw2d.parts.ScrollableThumbnail;
import org.eclipse.gef.DefaultEditDomain;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.gef.KeyStroke;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.palette.CreationToolEntry;
import org.eclipse.gef.palette.PaletteDrawer;
import org.eclipse.gef.palette.PaletteGroup;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.palette.SelectionToolEntry;
import org.eclipse.gef.palette.ToolEntry;
import org.eclipse.gef.requests.SimpleFactory;
import org.eclipse.gef.ui.parts.ContentOutlinePage;
import org.eclipse.gef.ui.parts.GraphicalEditorWithPalette;
import org.eclipse.gef.ui.parts.TreeViewer;
import org.eclipse.jface.resource.ImageDescriptor;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;

import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import org.eclipse.ui.IEditorPart;

import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.objectweb.petals.admin.actions.EsbAction;
import org.objectweb.petals.admin.adapter.EsbAdapter;
import org.objectweb.petals.admin.editparts.MyEditPartFactory;
import org.objectweb.petals.admin.editparts.tree.TreeEditPartFactory;
import org.objectweb.petals.admin.models.AgentModel;
import org.objectweb.petals.admin.models.ComponentModel;
import org.objectweb.petals.admin.models.ContentsEsbModel;
import org.objectweb.petals.admin.models.ContentsModel;
import org.objectweb.petals.admin.models.ContentsServiceContainerModel;
import org.objectweb.petals.admin.models.EsbModel;
import org.objectweb.petals.admin.models.ServiceContainerDirectoryModel;
import org.objectweb.petals.admin.states.State;
import org.objectweb.petals.admin.tools.BuildModels;
import org.objectweb.petals.admin.tools.BuildStatesXML;
import org.objectweb.petals.admin.tools.ImageDatas;
import org.objectweb.petals.admin.tools.MyContextMenuProvider;



/**
 * @author Maxime Carpentier
 * Plug-in Editor
 */

public class EsbEditor extends GraphicalEditorWithPalette  {
	public ArrayList StateList;
	private EsbModel esb = new EsbModel(this);
	private ContentsModel contentsModel;
	private EsbAdapter esbcontroller;
	private ServiceContainerDirectoryModel scdm = new ServiceContainerDirectoryModel(this);
	public String resourceFile;
	
	ContentsServiceContainerModel top = new ContentsServiceContainerModel(esb,scdm,true);
	ContentsServiceContainerModel bottom = new ContentsServiceContainerModel(esb,scdm,false);
	
	public ArrayList getStateList() {
		return StateList;
	}
	
	/**
	 * @param name
	 * @return the named state
	 */
	public State getState(String name) {
		for (int i = 0;i<StateList.size();i++) {
			if (((State) getStateList().get(i)).getName().equalsIgnoreCase(name)) { return ((State) getStateList().get(i)); }
		}
		return null;
	}
	/**
	 * class used to create the TreeView and the Thumbnail view
	 *
	 */
	class MyContentOutlinePage extends ContentOutlinePage {
		
		private SashForm sash;
		private ScrollableThumbnail thumbnail;
		private DisposeListener disposeListener;
		
		
		public MyContentOutlinePage() {
			super(new TreeViewer());
		}
		
		/**
		 * 
		 * @see org.eclipse.ui.part.IPage#createControl(org.eclipse.swt.widgets.Composite)
		 * implements
		 */
		public void createControl(Composite parent) {
			sash = new SashForm(parent, SWT.VERTICAL);
			
			getViewer().createControl(sash);
			getViewer().setEditDomain(getEditDomain());
			getViewer().setEditPartFactory((EditPartFactory) new TreeEditPartFactory());
			/* Tree model will be ServiceContainerDirectoryModel */
			getViewer().setContents(scdm);
			
			getSelectionSynchronizer().addViewer(getViewer());
			
			
			
			/* Thmubnail view s */
			Canvas canvas = new Canvas(sash, SWT.BORDER);
			
			LightweightSystem lws = new LightweightSystem(canvas);
			
			thumbnail = new ScrollableThumbnail((Viewport) ((ScalableRootEditPart) getGraphicalViewer().getRootEditPart()).getFigure());
			thumbnail.setSource(((ScalableRootEditPart) getGraphicalViewer().getRootEditPart()).getLayer(LayerConstants.PRINTABLE_LAYERS));
			lws.setContents(thumbnail);
			
			
			disposeListener = new DisposeListener() {
				public void widgetDisposed(DisposeEvent e) {
					if (thumbnail != null) {
						thumbnail.deactivate();
						thumbnail = null;
					}
				}
			};
			
			getGraphicalViewer().getControl().addDisposeListener(disposeListener);
			
			
			
		}
		
		public Control getControl() {
			
			return sash;
		}
		/**
		 * 
		 * @see org.eclipse.ui.part.IPage#dispose()
		 */
		public void dispose() {
			getSelectionSynchronizer().removeViewer(getViewer());
			super.dispose();
			if (getGraphicalViewer().getControl() != null
					&& !getGraphicalViewer().getControl().isDisposed())
				getGraphicalViewer().getControl().removeDisposeListener(disposeListener);
			
			super.dispose();
			
		}
	}
	
	public EsbEditor() {
		setEditDomain(new DefaultEditDomain(this));
	}
	
	/**
	 *  create global run and stop actions 
	 */
	public void createActions()
	{
		super.createActions();
		
	}
	
	/**
	 *  Load and set up the palette
	 */
	protected PaletteRoot getPaletteRoot() {
		PaletteRoot root = new PaletteRoot();
		
		PaletteGroup toolGroup = new PaletteGroup("palette");
		
		ToolEntry tool = new SelectionToolEntry();
		toolGroup.add(tool);
		root.setDefaultEntry(tool); 
		
		PaletteDrawer drawer = new PaletteDrawer("add...");
		
		ImageDescriptor descriptor = ImageDatas._component_small_menu;
		CreationToolEntry creationEntry =  new CreationToolEntry("Component", "add a component", new SimpleFactory(ComponentModel.class),descriptor, descriptor);
		drawer.add(creationEntry);
		
		ImageDescriptor descriptor2 = ImageDatas._agent_small_menu;
		CreationToolEntry creationEntry2 =  new CreationToolEntry("System service", "add a system service", new SimpleFactory(AgentModel.class),descriptor2, descriptor2);
		drawer.add(creationEntry2);
		
		root.add(toolGroup);
		root.add(drawer);
		
		return root;
	}
	
	/**
	 * Initialize viewer and gef models
	 * @see org.eclipse.gef.ui.parts.GraphicalEditor#initializeGraphicalViewer()
	 */
	protected void initializeGraphicalViewer() {
		IFile file = ((FileEditorInput) getEditorInput()).getFile();
		resourceFile =file.getLocation().toOSString();
		BufferedReader br;
		String server=" ";
		try {
			br = new BufferedReader(new InputStreamReader(new FileInputStream(resourceFile)));
			server = br.readLine();
		    esb.setServer(server);
		} catch (FileNotFoundException e3) {
			e3.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
			
		try {
			StateList = BuildStatesXML.build(this.getClass().getResource("/conf/states.xml"),this);

		} catch (MalformedURLException e1) {
			e1.printStackTrace();
		} catch (Exception e1) {
			e1.printStackTrace();
		}

		/* Initialize images */
		esbcontroller = new EsbAdapter();
		esbcontroller.setEsbeditor(this);
		GraphicalViewer viewer = getGraphicalViewer();
		scdm.setTop(top);
		scdm.setBottom(bottom);

		contentsModel = new ContentsModel();
		

		ContentsEsbModel mid = new ContentsEsbModel();

		viewer.setContextMenu(new MyContextMenuProvider(getActionRegistry(),viewer));
		
		
		contentsModel.addChild(top);
		contentsModel.addChild(mid);
		contentsModel.addChild(bottom);
		
		
		mid.addChild(esb);
		BuildModels bm = new BuildModels();
		
		connect(server);
		


	}
	
	public void connect(String server) {
			clearAll();
			BuildModels bm = new BuildModels();
			
			bm.build(scdm,getSite().getWorkbenchWindow(),esbcontroller,server);
			
			for(int i = 0;i < bm.getChildren().size();i++)	{
				if (i % 2 == 0) {
					top.addChild(bm.getChildren().get(i));
				}
				else {
					bottom.addChild(bm.getChildren().get(i));
				}
			}
			getGraphicalViewer().setContents(contentsModel);
	}
	
	public void clearAll() {
			while (top.getChildren().size() > 0)	{
				top.removeChild(top.getChildren().get(0));
		}
			while (bottom.getChildren().size() > 0)	{
				bottom.removeChild(bottom.getChildren().get(0));
		}
	}
	/**
	 * 
	 * @param action
	 * register an EsbAction to ActionRegistry.EsbAction are actions performed on containers,components or agents
	 */
	public void addAction(EsbAction action) {
		getActionRegistry().registerAction(action);
		getSelectionActions().add(action.getId());
	}
	/**
	 * 
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class aClass)
	{
		if (aClass == IContentOutlinePage.class)
			return new MyContentOutlinePage();
		return super.getAdapter(aClass);
	}
	
	
	/**
	 * @see org.eclipse.ui.part.EditorPart#doSaveAs()
	 */
	public void doSaveAs() {
		
	}
	
	/**
	 * @see org.eclipse.ui.part.EditorPart#gotoMarker(org.eclipse.core.resources.IMarker)
	 */
	public void gotoMarker(IMarker marker) {
		
	}
	
	/**
	 *
	 * @see org.eclipse.ui.ISaveablePart#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void doSave(IProgressMonitor monitor) {
		getCommandStack().markSaveLocation();
	}
	
	/**
	 *
	 * @see org.eclipse.ui.ISaveablePart#isDirty()
	 * @return if changes has been made in the EditDomain
	 */
	public boolean isDirty() {
		return false;
	}
	
	/**
	 *
	 * @see org.eclipse.gef.commands.CommandStackListener#commandStackChanged(java.util.EventObject)
	 */
	public void commandStackChanged(EventObject event) {
		firePropertyChange(IEditorPart.PROP_DIRTY);
		super.commandStackChanged(event);
	}
	
	
	/**
	 * @see org.eclipse.ui.part.EditorPart#isSaveAsAllowed()
	 */

	public boolean isSaveAsAllowed() {
		return false;
	}
	
	/**
	 * @see org.eclipse.gef.ui.parts.GraphicalEditor#configureGraphicalViewer()
	 * configure main viewer and set up DEL key to delete action
	 */
	protected void configureGraphicalViewer() {
		super.configureGraphicalViewer();
		
		GraphicalViewer viewer = getGraphicalViewer();
		viewer.setEditPartFactory(new MyEditPartFactory());
		
		KeyHandler keyHandler = new KeyHandler();
		
		keyHandler.put(
				KeyStroke.getPressed(SWT.DEL, 127, 0),
				getActionRegistry().getAction(ActionFactory.DELETE.getId()));
		
		getGraphicalViewer().setKeyHandler(keyHandler);
		
	}
	
	/**
	 * @return viewer's width
	 */
	public int getWidth() {
		return getGraphicalViewer().getControl().getBounds().width;
	}

	/**
	 * @param server
	 */
	public void updateResourceFile(String server) {
		try {
			FileWriter fw = new FileWriter(resourceFile);
			fw.write(server);
			fw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	
	
	
}
