/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.actions;

import java.util.List;

import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IWorkbenchPart;
import org.objectweb.petals.admin.EsbEditor;
import org.objectweb.petals.admin.editparts.EditPartWithListener;
import org.objectweb.petals.admin.editparts.ServiceContainerEditPart;
import org.objectweb.petals.admin.models.AbstractModel;
import org.objectweb.petals.admin.models.ServiceContainerModel;

/**
 * 
 * @author Maxime Carpentier
 *
 * 
 */
public class EsbAction extends SelectionAction
{
	public static String ID = "default action";
	private String method;
	private String icon;
	private String name;
	
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}
	public String toString() {
		return name + " - " + method;
	}
	/**
	 * @param part
	 * @param method
	 * @param icon
	 * @param name
	 * @param text
	 */
	public EsbAction(IWorkbenchPart part,String method,String icon,String name, String text)
	{
		super(part);
		this.method = method;
		this.icon=icon;
		setId(name);
		setName(name);
		setText(text);
		setImageDescriptor(ImageDescriptor.createFromFile(EsbEditor.class,icon));
	}
	/**
	 * 
	 * @param part
	 */
	public EsbAction(IWorkbenchPart part)
	{
		super(part);
}
	/**
	 * @return whether the action is displayed or not : look up the EditPart corresponding to the mouse's position to see if this EditPart support actions
	 */
	protected boolean calculateEnabled()
	{
			List list = getSelectedObjects();
			if (list.size() == 1)
			{
				if (!(list.get(0) instanceof EditPartWithListener)) { return false; }
				EditPartWithListener obj = (EditPartWithListener) list.get(0);
				if ((obj.getEditPartType() == EditPartWithListener._COMPONENTS_EDIT_PART) || (obj.getEditPartType() == EditPartWithListener._TRANSPORT_EDIT_PART) || (obj.getEditPartType() == EditPartWithListener._SERVICE_CONTAINER_TITLE_EDIT_PART)) {
					obj = (EditPartWithListener) obj.getParent();
				}
				if ((obj.getEditPartType() != EditPartWithListener._AGENT_EDIT_PART) && (obj.getEditPartType() != EditPartWithListener._COMPONENT_EDIT_PART)) { return false ; }
				if (((AbstractModel) obj.getModel()).getState().getActions().containsKey(name)) {
					return true;
				}
				else 
					return false;
			}
			return false;
	}
	/**
	 * execute the action : retrieve the model corresponding to the spotted EditPart and invoke a method on it
	 */
	public void run()
	{
		List list = getSelectedObjects();
		if(list.size() == 1)
		{
			EditPartWithListener obj = (EditPartWithListener) list.get(0);
			String tmp = null;
			if ((obj.getEditPartType() == EditPartWithListener._COMPONENTS_EDIT_PART) || (obj.getEditPartType() == EditPartWithListener._TRANSPORT_EDIT_PART)|| (obj.getEditPartType() == EditPartWithListener._SERVICE_CONTAINER_TITLE_EDIT_PART)) {
				// spotted object should be a ServiceContainer
				obj = (EditPartWithListener) obj.getParent(); 
				tmp = new String(((ServiceContainerModel) (((ServiceContainerEditPart) obj).getModel())).getName());
			}
			else if ((obj.getEditPartType() == EditPartWithListener._AGENT_EDIT_PART) || (obj.getEditPartType() == EditPartWithListener._COMPONENT_EDIT_PART)) { 
				// spotted object is a component or an agent
				tmp =  ((ServiceContainerModel) ((ServiceContainerEditPart) obj.getParent().getParent()).getModel()).getName();
				tmp =  tmp + ":" + Integer.toString(((ServiceContainerModel) ((ServiceContainerEditPart) obj.getParent().getParent()).getModel()).getPort());
			}
			else {
				// spotted object is  a ServiceContainer
				tmp = new String(((ServiceContainerModel) (((ServiceContainerEditPart) obj).getModel())).getName());
			}
			((AbstractModel) obj.getModel()).invoke(method,tmp);
		}
	}

}