/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.editparts;

import java.beans.PropertyChangeEvent;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.gef.EditPolicy;
import org.objectweb.petals.admin.editpolicies.MyAgentEditPolicy;
import org.objectweb.petals.admin.models.AgentModel;
import org.objectweb.petals.admin.tools.AgentFigure;



/**
 * @author Maxime Carpentier
 * Controller of an agent model
 *
 */
public class AgentEditPart extends EditPartWithListener {

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	private Label label;

	
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 * initilaize the graphical representation of the model
	 */
	protected IFigure createFigure() {
		AgentModel model = (AgentModel) getModel();
		AgentFigure figure = new AgentFigure(model.getState());
		return figure;
	}
	
	
	/**
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 * called when model changes; it updates the model's figure when required
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(AgentModel.P_CONSTRAINT) || evt.getPropertyName().equals(AgentModel.P_NAME))
			refreshVisuals(); 
		if (evt.getPropertyName().equals(AgentModel.P_STATE))
			refreshFigure();
	}
	
	/**
	 *  update image up to the model's state 
	 */
	public void refreshFigure() {
		((AgentFigure) this.getFigure()).setImg(((AgentModel) getModel()).getState().getImage());
		((AgentFigure) this.getFigure()).repaint();
		((AgentFigure) this.getFigure()).revalidate();
	}
	public int getEditPartType() {
		return _AGENT_EDIT_PART;
	}
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		    installEditPolicy(EditPolicy.COMPONENT_ROLE, new MyAgentEditPolicy());
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 * update the text of the label up to model's name property
	 */
	protected void refreshVisuals() {
			((AgentFigure) this.figure).setName(((AgentModel) getModel()).getName());
	}


}
