/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.editparts;

import java.beans.PropertyChangeEvent;
import java.util.List;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FlowLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Panel;
import org.eclipse.draw2d.ScrollPane;
import org.eclipse.gef.EditPolicy;
import org.eclipse.swt.graphics.Color;
import org.objectweb.petals.admin.editpolicies.ComponentsLayoutEditPolicy;
import org.objectweb.petals.admin.models.ComponentsModel;


/**
 * @author Maxime Carpentier
 * Controller of a "components" model : it can contains components
 *
 */
public class ComponentsEditPart extends EditPartWithListener {

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	private Panel components;
	private  Figure view  = new Figure();
	private ScrollPane scrollpane;

	public int getEditPartType() {
		return _COMPONENTS_EDIT_PART;
	}
	/**
	 * @return the contents of the scrollpane
	 */
	public IFigure getContentPane() {
		return components;
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 * initilaize the graphical representation of the model : set up the main panel with a vertical scrollbar
	 */
	protected IFigure createFigure() {
		ComponentsModel model = (ComponentsModel) getModel();
		scrollpane = new ScrollPane();
		FlowLayout fl1 = new FlowLayout(true);
		fl1.setMajorAlignment(FlowLayout.ALIGN_LEFTTOP);
		fl1.setMajorSpacing(ComponentsModel.SPACING);
		fl1.setMinorSpacing(ComponentsModel.SPACING);
		view.setLayoutManager(fl1);
		scrollpane.setContents(view);
	
		components = new Panel();
		components.setPreferredSize(model.CalculateViewSize().width-(ComponentsModel.BORDER_SPACING*2)-scrollpane.getVerticalScrollBar().getPreferredSize().width,model.CalculateViewSize().height-(ComponentsModel.BORDER_SPACING*2));
		view.add(components);
		FlowLayout fl = new FlowLayout(false);
		components.setLayoutManager(fl);
		scrollpane.setPreferredSize(ComponentsModel.CalculateInitialContainerSize());

		scrollpane.setBackgroundColor(new Color(null, 94,94,169));
		scrollpane.setOpaque(true);


		return scrollpane;
	}
	

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new ComponentsLayoutEditPolicy());
	}

	/**
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 * called when model changes; it updates the model's figure when required
	 */
	  public void propertyChange(PropertyChangeEvent evt) {
	    if (evt.getPropertyName().equals(ComponentsModel.P_CONSTRAINT)) {
	      refreshVisuals(); 
	    }
	    else if (evt.getPropertyName().equals(ComponentsModel.P_CHILDREN)) {
	    	UpdateServiceContainerView();
		    	refreshVisuals();
	    	refreshChildren();
	    }
	  }
	  
	/**
	 * Calculate and set the size of the scrollpane's contents regarding the number of components
	 *
	 */
	private void UpdateServiceContainerView() {
		((ComponentsModel)getModel()).UpdateViewSize();
		components.setPreferredSize(((ComponentsModel)getModel()).CalculateViewSize().width-(ComponentsModel.BORDER_SPACING*2)-scrollpane.getVerticalScrollBar().getPreferredSize().width,((ComponentsModel)getModel()).CalculateViewSize().height-(ComponentsModel.BORDER_SPACING*2));
		getFigure().setPreferredSize(ComponentsModel.CalculateInitialContainerSize());
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
	/*	Rectangle constraint = ((ComponentsModel) getModel()).getConstraint();
		((GraphicalEditPart) getParent()).setLayoutConstraint(
			this,
			getFigure(),
			constraint);*/
	}

	protected List getModelChildren() {
		return ((ComponentsModel) getModel()).getChildren();
	}
	
	/**
	 * @return Returns the scrollpane.
	 */
	public ScrollPane getScrollpane() {
		return scrollpane;
	}
}