/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.editparts;

import java.beans.PropertyChangeEvent;
import java.util.List;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FlowLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.Request;
import org.objectweb.petals.admin.models.ServiceContainerModel;
import org.objectweb.petals.admin.tools.ImageDatas;
import org.objectweb.petals.admin.tools.ServicesContainerBorder;

/**
* @author Maxime Carpentier
* */
public class ServiceContainerEditPart extends EditPartWithListener {
	public static final String EDITPARTTYPE = "_container";
	private String name = "_myServiceContainer";

	public int getEditPartType() {
		return _SERVICE_CONTAINER_EDIT_PART;
	}
	
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 * initilaize the graphical representation of the model
	 */
	protected IFigure createFigure() {
		Figure view  = new Figure();
	
		FlowLayout fl1 = new FlowLayout(false);
		fl1.setMajorAlignment(FlowLayout.ALIGN_CENTER);
		fl1.setStretchMinorAxis(true);
		fl1.setMajorSpacing(0);
		fl1.setMinorSpacing(0);
		view.setLayoutManager(fl1);
	
		//border
		ServicesContainerBorder border = new ServicesContainerBorder();
		
		border.setBottom(ImageDatas._container_bottom);
		border.setLeft(ImageDatas._container_left);
		border.setRight(ImageDatas._container_right);
		border.setTop(ImageDatas._container_top);
		
		border.setBottom_left(ImageDatas._container_bottom_left);
		border.setBottom_right(ImageDatas._container_bottom_right);
		border.setTop_left(ImageDatas._container_top_left);
		border.setTop_right(ImageDatas._container_top_right);
		
		view.setBorder(border);
		return view;
		
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	protected List getModelChildren() {
		return ((ServiceContainerModel) getModel()).getChildren();
	}



	/**
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 * called when model changes; it updates the model's figure when required
	 */
	  public void propertyChange(PropertyChangeEvent evt) {
	    if (evt.getPropertyName().equals(ServiceContainerModel.P_CONSTRAINT)) {
	      refreshVisuals(); 
	    }
	    else if (evt.getPropertyName().equals(ServiceContainerModel.P_CHILDREN)) {
		//    refreshVisuals();
	    	refreshChildren();
	    }
	    else if (evt.getPropertyName().equals(ServiceContainerModel.P_SOURCE_CONNECTION)) {
	    	refreshSourceConnections();
	    }
	    else if (evt.getPropertyName().equals(ServiceContainerModel.P_TARGET_CONNECTION)) {
	    	refreshTargetConnections();
	    }
	  }
	  

	/**)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 * Updates the position of the figure regarding the model
	 */
	protected void refreshVisuals() {
	/*	Rectangle constraint = ((ServiceContainerModel) getModel()).getConstraint();
		((GraphicalEditPart) getParent()).setLayoutConstraint(
			this,
			getFigure(),
			constraint);*/
	}

	/**
	 * 
	 * @param connection
	 * @return
	 */
	  public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
	    return new ChopboxAnchor(getFigure());
	  }

	  /**
	 * @param connection
	 * @return
	 */
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
	    return new ChopboxAnchor(getFigure());
	  }

	  /**
	 * @param request
	 * @return
	 */
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
	    return new ChopboxAnchor(getFigure());
	  }

	  /**
	 * @param request
	 * @return
	 */
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
	    return new ChopboxAnchor(getFigure());
	  }

	  /**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
	 */
	protected List getModelSourceConnections() {
	    return ((ServiceContainerModel)getModel()).getModelSourceConnections();
	  }

	  /**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
	 */
	protected List getModelTargetConnections() {
	    return ((ServiceContainerModel)getModel()).getModelTargetConnections();
	  }

	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
	}
}