/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.editparts;

import java.beans.PropertyChangeEvent;
import java.util.List;

import org.eclipse.draw2d.FlowLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.ImageFigure;
import org.eclipse.draw2d.ScrollPane;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPolicy;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.objectweb.petals.admin.editpolicies.TransportLayoutEditPolicy;
import org.objectweb.petals.admin.models.ComponentsModel;
import org.objectweb.petals.admin.models.TransportModel;
import org.objectweb.petals.admin.tools.ImageDatas;
import org.objectweb.petals.admin.tools.TransportBorder;

/**
* @author Maxime Carpentier
*/
public class TransportEditPart extends EditPartWithListener {
	public static final String EDITPARTTYPE = "_transport";
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	private ImageFigure transport;
	private  ImageFigure view  = new ImageFigure();
	private ScrollPane scrollpane;

	/**
	 * @see org.objectweb.petals.admin.editparts.EditPartWithListener#getEditPartType()
	 */
	public int getEditPartType() {
		return _TRANSPORT_EDIT_PART;
	}
	
	/**
	 * @see org.eclipse.gef.GraphicalEditPart#getContentPane()
	 */
	public IFigure getContentPane() {
		return transport;
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 * initilaize the graphical representation of the model
	 */
	protected IFigure createFigure() {
		TransportModel model = (TransportModel) getModel();
		scrollpane = new ScrollPane();
		FlowLayout fl1 = new FlowLayout(true);
		fl1.setMajorAlignment(FlowLayout.ALIGN_LEFTTOP);
		fl1.setMajorSpacing(ComponentsModel.SPACING);
		fl1.setMinorSpacing(ComponentsModel.SPACING);
		view.setLayoutManager(fl1);
			Image img = ((TransportModel) getModel()).getImg();
			// border
		TransportBorder border = new TransportBorder();
		border.setTop_left(model.getImgborderleft());
		border.setTop_right(model.getImgborderright());
			scrollpane.setBorder(border);
		Image cont =ImageDatas._transport;
		ImageData temp = cont.getImageData().scaledTo(model.CalculateInitialContainerSize().width , img.getBounds().height);
		transport = new ImageFigure();
		transport.setBounds(new Rectangle(transport.getLocation(),new Dimension(model.CalculateViewSize().width-(TransportModel.BORDER_SPACING*2)-scrollpane.getVerticalScrollBar().getPreferredSize().width,model.CalculateViewSize().height-(TransportModel.BORDER_SPACING*2))));
		transport.setPreferredSize(model.CalculateViewSize().width-(TransportModel.BORDER_SPACING*2)-scrollpane.getVerticalScrollBar().getPreferredSize().width,model.CalculateViewSize().height-(TransportModel.BORDER_SPACING*2));
		FlowLayout fl = new FlowLayout(false);
			fl.setMinorAlignment(FlowLayout.ALIGN_CENTER);
		fl.setMajorAlignment(FlowLayout.ALIGN_CENTER);
		transport.setLayoutManager(fl);
		transport.setImage(new Image(null,temp));
		scrollpane.setContents(transport);
		scrollpane.setPreferredSize(model.CalculateInitialContainerSize());

		return scrollpane;
	}
	

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.LAYOUT_ROLE, new TransportLayoutEditPolicy());
	}

	/**
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 * called when model changes; it updates the model's figure when required
	 */
	  public void propertyChange(PropertyChangeEvent evt) {
	    if (evt.getPropertyName().equals(ComponentsModel.P_CONSTRAINT)) {
	      refreshVisuals(); 
	    }
	    else if (evt.getPropertyName().equals(ComponentsModel.P_CHILDREN)) {
	    	UpdateServiceContainerView();
		    	refreshVisuals();
	    	refreshChildren();
	    }
	  }
	  

	/**
	 * Calculate and set the size of the scrollpane's contents regarding the number of agents
	 */
	private void UpdateServiceContainerView() {
		TransportModel model = (TransportModel) getModel();
		model.UpdateViewSize();
		transport.setPreferredSize(model.CalculateViewSize().width-(TransportModel.BORDER_SPACING*2)-scrollpane.getVerticalScrollBar().getPreferredSize().width,model.CalculateViewSize().height-(TransportModel.BORDER_SPACING*2));

		getFigure().setPreferredSize(model.CalculateInitialContainerSize());
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals() {
	/*	Rectangle constraint = ((TransportModel) getModel()).getConstraint();
		((GraphicalEditPart) getParent()).setLayoutConstraint(
			this,
			getFigure(),
			constraint);*/
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 */
	protected List getModelChildren() {
		return ((TransportModel) getModel()).getChildren();
	}
	
}