/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.models;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.objectweb.petals.admin.states.State;


/**
* @author Maxime Carpentier
 * Abstract model, have to be implemented by all models that belongs to the MVC
 *
 */
abstract public class AbstractModel implements IPropertySource {

	private PropertyChangeSupport listeners = new PropertyChangeSupport(this);
	public State state;
	
	public void addPropertyChangeListener(PropertyChangeListener listener) {
		listeners.addPropertyChangeListener(listener);
	}

	/**
	 * @param propName
	 * @param oldValue
	 * @param newValue
	 */
	public void firePropertyChange(
		String propName,
		Object oldValue,
		Object newValue) {

		listeners.firePropertyChange(propName, oldValue, newValue);
	}
	/**
	 * @return the Id of the model. each model type has an unique Id
	 */
	public String getId() {
		return "_abstractModel";
	}
	/**
	 * @return the current state
	 */
	public State getState() {
		return null;
	}

	/**
	 * @param listener
	 * unhook model's listener
	 */
	public void removePropertyChangeListener(PropertyChangeListener listener) {
		listeners.removePropertyChangeListener(listener);
	}
	
	  public Object getEditableValue() {
	  	return this;
	  }

	  /**
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 * usefull to provides properties to the properties view
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {

	    return new IPropertyDescriptor[0];
	  }

	  /**
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object id) {
	    return null;
	  }

	  public boolean isPropertySet(Object id) {
	    return false;
	  }

	  public void resetPropertyValue(Object id) {
	  }

	  public void setPropertyValue(Object id, Object value) {
	  }
	/**
	 * @param method
	 * @param container
	 */
	public void invoke(String method, String container) {
	}

	
}
