/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.models;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import org.objectweb.petals.admin.adapter.EsbAdapter;
import org.objectweb.petals.admin.states.State;


public class AgentModel extends AbstractModel {

	public static final String P_CONSTRAINT = "_constraint";
	public static final String P_STATE = "_agentState";
	public static final String P_INVOKE = "_AgentInvoke";
	public static final String P_FOCUS = "_focus";
	public static final String P_NOM = "_agent";
	public static final String P_TYPE = "_type";
	private final String P_ID = "_agentModell";
	private String name ="_myAgent";
	private String type ="_myType";
	private Rectangle constraint;
	public static final String P_NAME = "_name";
	public State state;
	private String container;
	private EsbAdapter esbcontroller;
	/**
	 * @return Returns the container.
	 */
	public String getContainer() {
		return container;
	}
	/**
	 * @param container The container to set.
	 */
	public void setContainer(String container) {
		this.container = container;
	}
	
	/**
	 * @see org.objectweb.petals.admin.models.AbstractModel#invoke(java.lang.String, java.lang.String)
	 * @param method The method to invoke
	 * @param container The name of the agent's container
	 */
	public void invoke(String method,String container) {
		this.container = container;
		firePropertyChange(P_INVOKE, this, method);
	}
	
	/**
	 * @param esbcontroller
	 */
	public AgentModel(EsbAdapter esbcontroller) {
		this.addPropertyChangeListener(esbcontroller);
		this.esbcontroller = esbcontroller;
		//default state
		State state1 = esbcontroller.getEsbeditor().getState("esb.state.agentStopped");
		this.state = state1;

		
	}
	/**
	 * @see org.objectweb.petals.admin.models.AbstractModel#getState()
	 * @return the current state
	 */
	public State getState() {
		return state;
	}
	/**
	 * @param state
	 */
	public void setState(State state) {
		this.state = state;
		firePropertyChange(P_STATE, this, state);
	}
	  /**
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 * called to provide properties to the properties view
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
	  	firePropertyChange(P_FOCUS, null, null);
	    IPropertyDescriptor[] descriptors =
	      new IPropertyDescriptor[] {
	    		new TextPropertyDescriptor(P_NAME, "name"),new TextPropertyDescriptor(P_TYPE, "type"),new TextPropertyDescriptor(P_STATE, "status")};
	    return descriptors;

	  }
		/**
		 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
		 */
		public Object getPropertyValue(Object id) {
			if (id.equals(P_NAME)) {
				return name;
			}
			if (id.equals(P_TYPE)) {
				return type;
			}
			if (id.equals(P_STATE)) {
				return getState().getId().toLowerCase();
			}
			return null;
		}

		/**
		 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
		 */
		public boolean isPropertySet(Object id) {
			if (id.equals(P_NAME))
				return true;
			if (id.equals(P_TYPE))
				return true;
			if (id.equals(P_STATE))
				return true;
			else
				return false;
		}

		/**
		 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
		 */
		public void setPropertyValue(Object id, Object value) {
			if (id.equals(P_NAME)) {
				setName((String) value);
			}
			else if (id.equals(P_TYPE)) {
				setType((String) value);
			}
		}
		
	/**
	 * @param caption The name of the Agent
	 */
	public AgentModel(String caption) {
		this.setName(caption);
	}
	
	/**
	 * @param esbcontroller
	 * @param string
	 */
	public AgentModel(EsbAdapter esbcontroller, String string) {
		this.addPropertyChangeListener(esbcontroller);
		this.esbcontroller = esbcontroller;
		//default state
		State state1 = esbcontroller.getEsbeditor().getState("esb.state.agentStopped");
		this.state = state1;
		setName(string);

	}
	public Rectangle getConstraint() {
		return constraint;
	}

	public void setConstraint(Rectangle rect) {
		constraint = rect;
		firePropertyChange(P_CONSTRAINT, null, constraint);
	}
	/**
	 * @return Returns the p_NAME.
	 */
	public static String getP_NAME() {
		return P_NAME;
	}
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
		firePropertyChange(P_NAME, null, name);
	}
	public String getType() {
		return type;
	}
	public void setType(String type) {
		this.type = type;
	}
	/**
	 * @param string
	 */
	public void setState(String string) {

		State state1 = esbcontroller.getEsbeditor().getState("esb.state.agent"+string);
		setState(state1);
	}
}
