/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.models;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import org.objectweb.petals.admin.EsbEditor;
import org.objectweb.petals.admin.tools.MsgBox;

/**
* @author Maxime Carpentier
*/
public class EsbModel extends AbstractModel implements PropertyChangeListener {
	public static final String P_CONSTRAINT = "_constraint";
	public static final String P_SIZE = "_size";
	public static final String P_SERVER = "_esbServer";
	private int topSize,bottomSize = 0;
	private String text = "E S B";
	private EsbEditor esbeditor;

	// ESB properties
	private String server = " ";
	
	private Rectangle constraint;
	
	// connection lines
	  public static final String P_SOURCE_CONNECTION = "_source_connection";
	  public static final String P_TARGET_CONNECTION = "_target_connection";

	  private List sourceConnections = new ArrayList();
	  private List targetConnections = new ArrayList();

	  public void addSourceConnection(Object connx) {
	    sourceConnections.add(connx);
	    firePropertyChange(P_SOURCE_CONNECTION, null, null);
	  }

	  public void addTargetConnection(Object connx) {
	    targetConnections.add(connx);
	    firePropertyChange(P_TARGET_CONNECTION, null, null);
	  }

	  public List getModelSourceConnections() {
	    return sourceConnections;
	  }

	  public List getModelTargetConnections() {
	    return targetConnections;
	  }

	  public void removeSourceConnection(Object connx) {
	    sourceConnections.remove(connx);
	    firePropertyChange(P_SOURCE_CONNECTION, null, null);
	  }

	  public void removeTargetConnection(Object connx) {
	    targetConnections.remove(connx);
	    firePropertyChange(P_TARGET_CONNECTION, null, null);
	  }

	  
	  public EsbModel(EsbEditor esbeditor) {
	  	this.esbeditor = esbeditor;
	  }
	  
	public String getText() {
		return text;
	}

	public void setText(String text) {
		this.text = text;
	}

	public Rectangle getConstraint() {
		return constraint;
	}

	public void setConstraint(Rectangle rect) {
		constraint = rect;
		firePropertyChange(P_CONSTRAINT, null, constraint);
	}
	
	/**
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 * called to provide properties to the properties view
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		IPropertyDescriptor[] descriptors =
			new IPropertyDescriptor[] {
				 new TextPropertyDescriptor(P_SERVER, "server (host:port)")};
		return descriptors;

	}

	/**
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object id) {
		if (id.equals(P_SERVER)) {
			return server;
		}
		return null;
	}

	/**
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(Object id) {
		if (id.equals(P_SERVER))
			return true;
		else
			return false;
	}

	/**
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object id, Object value) {
		if (id.equals(P_SERVER)) {
	        StringTokenizer stmp = new StringTokenizer((String) value,":");
	        if (stmp.countTokens() != 2) { 
				MsgBox msg = new MsgBox(esbeditor.getSite().getWorkbenchWindow());
				msg.display("Wrong server entry (usage: host:port)","error");
				return;
	        }
	        String host = stmp.nextToken();
	        String port = stmp.nextToken();
			setServer((String) value);
			esbeditor.connect(server);
			esbeditor.updateResourceFile(server);
		}
	}
	
	public String getServer() {
		return server;
	}
	public void setServer(String server) {
		if (server != null) { this.server = server; }
		else { server = " "; }
	}
	/**
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 * called when a servicecontainermodel is changed, it modifies the model's size regarding the number of containers
	 */
	public void propertyChange(PropertyChangeEvent evt) {
	    if ((evt.getPropertyName().equals(ContentsServiceContainerModel.P_CHILDREN)) && (evt.getOldValue() != null)) {
	    	if (((ContentsServiceContainerModel) evt.getOldValue()).getPosition().equals(ContentsServiceContainerModel.P_TOP)) {
	    		topSize = ((ContentsServiceContainerModel) evt.getOldValue()).getChildren().size();
	    	}
	    	else if (((ContentsServiceContainerModel) evt.getOldValue()).getPosition().equals(ContentsServiceContainerModel.P_BOTTOM)) {
	    		bottomSize = ((ContentsServiceContainerModel) evt.getOldValue()).getChildren().size();
	    	}
			firePropertyChange(P_SIZE, null, null);
		}
	}
	public int getBottomSize() {
		return bottomSize;
	}
	public int getTopSize() {
		return topSize;
	}
}
