/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.models;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import org.objectweb.petals.admin.adapter.EsbAdapter;

import org.objectweb.petals.description.ContainerDescription;
/**
* @author Maxime Carpentier
*/
public class ServiceContainerModel extends AbstractModel {
	public static final String P_CONSTRAINT = "_constraint";
	public static final String P_CHILDREN = "_children";
	public static final String P_FOCUS = "_focus";
	public static final String P_NAME = "_name";
	public static final String P_HOST = "_host";
	public static final String P_PORT = "_port";
	public static final String P_TITLE = "_title";
	public final String P_ID = "_serviceContainerModel";
	public static final int BORDER_SPACING=4; //bords du container
	private String name = "ServiceContainer";
	private String host = "myHost";
	private int port = 9999;
	private String container;
	private Rectangle constraint;
	private IWorkbenchWindow window;
	private List children = new ArrayList(); 
	private ServiceContainerDirectoryModel serviceDirectoryModel;
	private EsbAdapter esbcontroller;
	private ContainerDescription cd;
	
	/**
	 * @return Returns the host.
	 */
	public String getHost() {
		return host;
	}
	/**
	 * @param host The host to set.
	 */
	public void setHost(String host) {
		this.host = host;
	}

	
	public String getId() {
		return P_ID;
	}
	/**
	 * @param child
	 */
	public void addChild(Object child) {
		children.add(child); 
		firePropertyChange(P_CHILDREN, null, null);
	}
	/**
	 * @param arg
	 * @param child
	 */
	public void moveChild(int arg,Object child) {
			children.remove(child);
			children.add(arg,child);
			firePropertyChange(P_CHILDREN, null, null);
}
	
	/**
	 * @param state
	 * @param sdm
	 * @param window
	 * @param esbcontroller
	 */
	public ServiceContainerModel(ContainerDescription cd,ServiceContainerDirectoryModel sdm,EsbAdapter esbcontroller) {
		 serviceDirectoryModel = sdm;
		 this.cd = cd;
		 this.name = cd.getName();
		 this.host = cd.getHost();
		 this.port = cd.getAdminPort();
		 this.esbcontroller = esbcontroller;
		 ServiceContainerTitleModel title = new ServiceContainerTitleModel(this);
		 addChild(title);
		 this.addPropertyChangeListener(title);
		 this.addPropertyChangeListener(esbcontroller);
		 
		addChild(new ComponentsModel(sdm));
		addChild(new TransportModel(sdm));
	}
	
	/**
	 * @param editpart
	 */
	public void selectionChanged(Object editpart) {
		firePropertyChange(P_FOCUS, editpart, null);
	}
	
	public List getChildren() {
		return children;
	}

	public Object getChild(String name, String type) {
		if (type.equalsIgnoreCase("system")) {
			for (int i = 0;i < ((TransportModel) children.get(2)).getChildren().size();i++) {
				if (((AgentModel) ((TransportModel) children.get(2)).getChildren().get(i)).getName().equalsIgnoreCase(name)) {
					return ((TransportModel) children.get(2)).getChildren().get(i);
				}
			}		
		}
		else {
			for (int i = 0;i < ((ComponentsModel) children.get(1)).getChildren().size();i++) {
				if (((ComponentModel) ((ComponentsModel) children.get(1)).getChildren().get(i)).getName().equalsIgnoreCase(name)) {
					return ((ComponentsModel) children.get(1)).getChildren().get(i);
				}
			}
		}
		return null;
	}
	/**
	 * @param child
	 */
	public void removeChild(Object child) {
		children.remove(child);
		firePropertyChange(P_CHILDREN, null, null);
	}

	public String getName() {
		return name;
	}
	
	public void setName(String text) {
		this.name = text;
	}

	public Rectangle getConstraint() {
		return constraint;
	}

	public void setConstraint(Rectangle rect) {
		constraint = rect;
		firePropertyChange(P_CONSTRAINT, null, constraint);
	}
	
	// connection lines stuffs
	  public static final String P_SOURCE_CONNECTION = "_source_connection";
	  public static final String P_TARGET_CONNECTION = "_target_connection";

	  private List sourceConnections = new ArrayList();
	  private List targetConnections = new ArrayList();

	  public void addSourceConnection(Object connx) {
	    sourceConnections.add(connx);
	    firePropertyChange(P_SOURCE_CONNECTION, null, null);
	  }

	  public void addTargetConnection(Object connx) {
	    targetConnections.add(connx);
	    firePropertyChange(P_TARGET_CONNECTION, null, null);
	  }

	  public List getModelSourceConnections() {
	    return sourceConnections;
	  }

	  public List getModelTargetConnections() {
	    return targetConnections;
	  }

	  public void removeSourceConnection(Object connx) {
	    sourceConnections.remove(connx);
	    firePropertyChange(P_SOURCE_CONNECTION, null, null);
	  }

	  public void removeTargetConnection(Object connx) {
	    targetConnections.remove(connx);
	    firePropertyChange(P_TARGET_CONNECTION, null, null);
	  }
	
	public IWorkbenchWindow getWindow() {
		return window;
	}
	public void setWindow(IWorkbenchWindow window) {
		this.window = window;
	}

	  /**
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 * called to provide properties to the properties view
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
	    IPropertyDescriptor[] descriptors =
	      new IPropertyDescriptor[] {
	    		new TextPropertyDescriptor(P_TITLE, "name"),new TextPropertyDescriptor(P_HOST, "host"),new TextPropertyDescriptor(P_PORT, "port")};
	    return descriptors;

	  }
		/**
		 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
		 */
		public Object getPropertyValue(Object id) {
			if (id.equals(P_TITLE)) {
				return getName();
			}
			if (id.equals(P_HOST)) {
				return getHost();
			}
			if (id.equals(P_PORT)) {
				return Integer.toString(getPort());
			}
			return null;
		}

		/**
		 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
		 */
		public boolean isPropertySet(Object id) {
			if (id.equals(P_TITLE))
				return true;
			else if (id.equals(P_HOST))
				return true;
			else if (id.equals(P_PORT))
				return true;
			else
				return false;
		}

		/**
		 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
		 */
		public void setPropertyValue(Object id, Object value) {
			if (id.equals(P_TITLE)) {
				setName((String) value);
			}
			if (id.equals(P_HOST)) {
				setHost((String) value);
			}
			if (id.equals(P_PORT)) {
				setPort(Integer.parseInt((String) value));
			}
		}

	/**
	 * @return Returns the esbcontroller.
	 */
	public EsbAdapter getEsbcontroller() {
		return esbcontroller;
	}
	/**
	 * @return Returns the container.
	 */
	public String getContainer() {
		return container;
	}
	public int getPort() {
		return port;
	}

	public void setPort(int port) {
		this.port = port;
	}
}
