/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.models;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IWorkbenchWindow;
import org.objectweb.petals.admin.tools.AgentFigure;
import org.objectweb.petals.admin.tools.ImageDatas;


/**
* @author Maxime Carpentier
 * This model can contains agent models
 *
 */
public class TransportModel extends AbstractModel {
	public static final String P_CONSTRAINT = "_constraint";
	public static final String P_CHILDREN = "_children";
	public static final String P_NAME = "_name";
	public final String P_ID = "_transportModel";
	public static final int BORDER_SPACING=0; //bords du container
	public static final int SPACING=8; // espacement inter services
	public static int P_INIT_ROWS=1;
	public static int P_ROWS=1;
	public static int P_COLUMNS=3;
	private String text = "transport";
	private Rectangle constraint;
	private IWorkbenchWindow window;
	private List children = new ArrayList(); 
	private ServiceContainerDirectoryModel serviceDirectoryModel;
	private Image img = ImageDatas._transport;
	private Image imgborderleft = ImageDatas._transport_left;
	private Image imgborderright = ImageDatas._transport_right;
	
	/**
	 * @see org.objectweb.petals.admin.models.AbstractModel#getId()
	 */
	public String getId() {
		return P_ID;
	}

	public Image getImg() {
	return img;
}
	/**
	 * @param child
	 */
	public void addChild(Object child) {
		children.add(child); 
		firePropertyChange(P_CHILDREN, null, null);
	}
	/**
	 * @param arg
	 * @param child
	 */
	public void moveChild(int arg,Object child) {
		children.remove(child);
		children.add(arg,child);
		firePropertyChange(P_CHILDREN, null, null);
	}
	/**
	 * @param sdm
	 */
	public TransportModel(ServiceContainerDirectoryModel sdm) {
		 serviceDirectoryModel = sdm;
	//	setConstraint(new Rectangle(new Point(0,0),CalculateInitialContainerSize()));
	}
	
	/**
	 * @param sdm
	 * @param window
	 */
	public TransportModel(ServiceContainerDirectoryModel sdm,IWorkbenchWindow window) {
		 serviceDirectoryModel = sdm;
		 this.window = window;
	//	setConstraint(new Rectangle(new Point(0,0),CalculateInitialContainerSize()));
	}
	
	/**
	 * @return the size of the container without any agent in it
	 */
	public Dimension CalculateInitialContainerSize() {
		int w,h;
		h = getImgborderleft().getBounds().height;
		w = ComponentsModel.CalculateInitialContainerSize().width-imgborderleft.getBounds().width-imgborderright.getBounds().width;
		return new Dimension(w,h);
	}
	/**
	 * @return the size the container should have regarding the current number of agent
	 */
	public Dimension CalculateViewSize() {
		int w,h;
		h = 2*BORDER_SPACING + ((P_ROWS - 1) * SPACING) + (P_ROWS * AgentFigure.P_HEIGHT);
		w = ComponentsModel.CalculateInitialContainerSize().width-imgborderleft.getBounds().width-imgborderright.getBounds().width;
		return new Dimension(w,h);
	}
	
	/**
	 * Update the number of rows
	 */
	public void UpdateViewSize() {
		int w,h,extra_rows;
			extra_rows = getChildren().size()>(P_INIT_ROWS*P_COLUMNS)?(int) (Math.ceil((double) (getChildren().size() - (P_COLUMNS*P_INIT_ROWS))/P_COLUMNS)):0;
			P_ROWS = P_INIT_ROWS + extra_rows;
	}
	/**
	 * @return The children
	 */
	public List getChildren() {
		return children;
	}

	/**
	 * @param child
	 */
	public void removeChild(Object child) {
		children.remove(child);
		serviceDirectoryModel.removeChild(child);
		firePropertyChange(P_CHILDREN, null, null);
	}

	public String getText() {
		return text;
	}
	
		public void setText(String text) {
		this.text = text;
	}

	public Rectangle getConstraint() {
		return constraint;
	}

	public void setConstraint(Rectangle rect) {
		constraint = rect;
		firePropertyChange(P_CONSTRAINT, null, constraint);
	}
	
	public IWorkbenchWindow getWindow() {
		return window;
	}
	public void setWindow(IWorkbenchWindow window) {
		this.window = window;
	}

	public Image getImgborderleft() {
		return imgborderleft;
	}
	public Image getImgborderright() {
		return imgborderright;
	}
	public void setImg(Image img) {
		this.img = img;
	}
}
