/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.tools;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.StringTokenizer;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.Binding;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;

import org.eclipse.ui.IWorkbenchWindow;
import org.objectweb.petals.admin.adapter.EsbAdapter;
import org.objectweb.petals.admin.models.AgentModel;
import org.objectweb.petals.admin.models.ComponentModel;
import org.objectweb.petals.admin.models.ComponentsModel;
import org.objectweb.petals.admin.models.ServiceContainerDirectoryModel;
import org.objectweb.petals.admin.models.ServiceContainerModel;
import org.objectweb.petals.admin.models.TransportModel;

import org.objectweb.petals.description.ContainerDescription;


/**
 * @author Maxime Carpentier
 */
public class BuildModels {
	public  List children = new ArrayList();
	private  InitialContext ic;
	private static final String CONTAINERS_REF = "containers";
	
	/**
	 * @return Returns the children.
	 */
	public  List getChildren() {
		return children;
	}

	public void build(ServiceContainerDirectoryModel scdm,IWorkbenchWindow window, EsbAdapter esbcontroller,String server)  {
				
		ClassLoader oldLoader = Thread.currentThread().getContextClassLoader();
		ClassLoader newLoader = this.getClass().getClassLoader();
		   
		Thread.currentThread().setContextClassLoader(newLoader);
		
		try {
			this.getClass().getClassLoader().loadClass("org.objectweb.petals.description.ContainerDescription");
		} catch (ClassNotFoundException e3) {
					e3.printStackTrace();
		}
		
		Properties p = new Properties();
		p.put(Context.INITIAL_CONTEXT_FACTORY,"fr.dyade.aaa.jndi2.client.NamingContextFactory");
		if (server == null) {
			MsgBox msg = new MsgBox(esbcontroller.getEsbeditor().getSite().getWorkbenchWindow());
			msg.display("click on the ESB figure and set the server description in the property view","configuration file is empty");
			return;
		}
		
        StringTokenizer stmp = new StringTokenizer(server,":");
        String host = stmp.nextToken();
        String port = stmp.nextToken();
		p.put("java.naming.factory.host", host);
		p.put("java.naming.factory.port", port);
		
		try {
			ic = new InitialContext(p);
		} catch (NamingException e2) {
			e2.printStackTrace();
			return;
		}
        String name = "" ;
        		try {
		        for(NamingEnumeration e = ic.listBindings(CONTAINERS_REF);e.hasMore();) {
				            Binding tmp = (Binding)e.next();
				            name  = tmp.getName();
				            ContainerDescription cd =  (ContainerDescription)tmp.getObject();
							
				            ServiceContainerModel scm = new ServiceContainerModel(cd,scdm,esbcontroller);
				            
							//****************JMX***************
				            //      The address of the connector server
				            JMXServiceURL url = new JMXServiceURL("service:jmx:rmi:///jndi/rmi://"+cd.getHost()+":"+cd.getAdminPort()+"/server");
				            //         The creation environment map, null in this case
				            Map creationEnvironment = null;
				            //         Create the JMXCconnectorServer
				            JMXConnector cntor = JMXConnectorFactory.newJMXConnector(url, creationEnvironment);
				            //         The connection environment map, null in this case
				            //         May contain - for example - user's credentials
				            Map connectionEnvironment = null;
				            //         Connect
				            cntor.connect(connectionEnvironment);
				            //         Obtain a "stub" for the remote MBeanServer
				            MBeanServerConnection mbsc = cntor.getMBeanServerConnection();
				            esbcontroller.AddMBeanServerConnections(mbsc,cd.getName()+":"+cd.getAdminPort());
				            //         Call the remote MBeanServer
				            String domain = mbsc.getDefaultDomain();


					        ObjectName an = new ObjectName("org.objectweb.petals:name=admin");
				            ObjectName components[] =(ObjectName[]) mbsc.getAttribute(an,"BindingComponents");
				            JMXServerListener listener = new JMXServerListener(mbsc,scm);
				            for (int i =0;i < components.length;i++) {
										//components name
							            ComponentModel cm = new ComponentModel(esbcontroller,components[i].getKeyProperty("name"));
							           cm.setType(components[i].getKeyProperty("type"));
							            //get components state
							            String state = (String) mbsc.getAttribute(components[i],"CurrentState");
							            cm.setState(esbcontroller.getEsbeditor().getState("esb.state.component"+state));
							            
							            //listener
							            listener.addComponent(components[i]);
							            
							            ((ComponentsModel) scm.getChildren().get(1)).addChild(cm);
				            }
				            components =(ObjectName[]) mbsc.getAttribute(an,"EngineComponents");
				            for (int i =0;i < components.length;i++) {
										//components name

							           ComponentModel cm = new ComponentModel(esbcontroller,components[i].getKeyProperty("name"));
								        cm.setType(components[i].getKeyProperty("type"));
								           
							            //get components state
							            String state = (String) mbsc.getAttribute(components[i],"CurrentState");
							            cm.setState(esbcontroller.getEsbeditor().getState("esb.state.component"+state));
							
							            //listener
							            listener.addComponent(components[i]);
							            
							            ((ComponentsModel) scm.getChildren().get(1)).addChild(cm);
				            }
				            components =(ObjectName[]) mbsc.getAttribute(an,"SystemServices");
				            for (int i =0;i < components.length;i++) {
										//components name
							            
							           AgentModel am = new AgentModel(esbcontroller,components[i].getKeyProperty("name"));
							            am.setType(components[i].getKeyProperty("type"));
							           
							            //get components state
							            String state = (String) mbsc.getAttribute(components[i],"CurrentState");
							            am.setState(esbcontroller.getEsbeditor().getState("esb.state.agent"+state));
							
							            //listener
							            listener.addComponent(components[i]);
							            
							            ((TransportModel) scm.getChildren().get(2)).addChild(am);
				            }
				children.add(scm);
		        }
		}
			catch (Exception e1)
			{
				MsgBox msg = new MsgBox(esbcontroller.getEsbeditor().getSite().getWorkbenchWindow());
				msg.display("Unable to connect to " + host+":" + port+"\n"+e1.getMessage(),"error");
			    e1.printStackTrace();
			}
	
        }
		
		
}
	
	

	
   

	
	
	
	
	
	
	
	
	
	
	
	
	
	



	
	 
