/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: 
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.admin.tools;

import org.eclipse.draw2d.AbstractBorder;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;

/**
 * @author Maxime Carpentier
 *
 * Custom border used with components
 */
public class ServicesContainerBorder extends AbstractBorder {
	private Image top_left,top,top_right,right,bottom_right,bottom,bottom_left,left;
	
	/**
	 * @see org.eclipse.draw2d.Border#getInsets(org.eclipse.draw2d.IFigure)
	 */
	public Insets getInsets(IFigure figure) {
		Rectangle rect = figure.getBounds();
		return new Insets(0,0,bottom.getBounds().height,right.getBounds().width);
	}

	/**
	 * @see org.eclipse.draw2d.Border#paint(org.eclipse.draw2d.IFigure, org.eclipse.draw2d.Graphics, org.eclipse.draw2d.geometry.Insets)
	 * paints the right and the bottom borders (there's no left and top borders)
	 */
	public void paint(IFigure figure, Graphics graphics, Insets insets) {
		Rectangle rect = getPaintRectangle(figure,insets);
			graphics.drawImage(top_right,rect.x+rect.width-top_right.getBounds().width,rect.y);
		graphics.drawImage(bottom_right,rect.x+rect.width-bottom_right.getBounds().width,rect.y+rect.height-bottom_right.getBounds().height);
		graphics.drawImage(bottom_left,rect.x,rect.y+rect.height-bottom_left.getBounds().height);
		
		
		ImageData temp = bottom.getImageData().scaledTo(rect.width-bottom_left.getBounds().width-bottom_right.getBounds().width,bottom.getBounds().height);
		graphics.drawImage(new Image(null,temp),rect.x+bottom_left.getBounds().width,rect.y+rect.height-bottom.getBounds().height);

		temp = right.getImageData().scaledTo(right.getBounds().width,rect.height-bottom_right.getBounds().height-top_right.getBounds().height);
		graphics.drawImage(new Image(null,temp),rect.x+rect.width-bottom_right.getBounds().width,rect.y+top_right.getBounds().height);

	}

	public void setBottom(Image bottom) {
		this.bottom = bottom;
	}
	public void setBottom_left(Image bottom_left) {
		this.bottom_left = bottom_left;
	}
	public void setBottom_right(Image bottom_right) {
		this.bottom_right = bottom_right;
	}
	public void setLeft(Image left) {
		this.left = left;
	}
	public void setRight(Image right) {
		this.right = right;
	}
	public void setTop(Image top) {
		this.top = top;
	}
	public void setTop_left(Image top_left) {
		this.top_left = top_left;
	}
	public void setTop_right(Image top_right) {
		this.top_right = top_right;
	}
}
