/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: InstallationContextImpl.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.component.context;

import java.util.List;

import javax.jbi.component.ComponentContext;
import javax.jbi.component.InstallationContext;

import org.w3c.dom.DocumentFragment;

/**
 * TODO InstallationContextImpl
 * @author Adrien LOUIS - EBM WebSourcing
 *
 */
public class InstallationContextImpl implements InstallationContext
{
    
    protected List classpathElements;
    
    protected boolean install;
    
    protected String componentClassName;
    
    protected String componentName;
    
    protected String installRoot;
    
    protected ComponentContextForInstallation context;
    
    protected DocumentFragment installationDescriptorExtension;
    
    /**
     * Constructor
     * @param componentClasspath the classpath for this component
     * @param install the component is installed or deinstalled
     * @param componentClassName the className of the component
     * @param componentName the name of the component
     * @param installRoot the path of the installation
     * @param context the context for this installation
     * @param installationDescriptorExtension installation descriptor (jbi.xml) extension 
     */ 
    public InstallationContextImpl(
            List componentClasspath, 
            boolean install,
            String componentClassName, 
            String componentName, 
            String installRoot, 
            ComponentContext context,
            DocumentFragment installationDescriptorExtension)
    {
        this.classpathElements 					= componentClasspath;
        this.install 							= install;
        this.componentClassName 				= componentClassName;
        this.componentName 						= componentName;
        this.installRoot 						= installRoot;
        this.context 							= new ComponentContextForInstallation(context);
        this.installationDescriptorExtension 	= installationDescriptorExtension;
    }
    
    /**
     * @see javax.jbi.component.InstallationContext#getClassPathElements()
     */
    public List getClassPathElements()
    {
        return classpathElements;
    }
    
    /**
     * @see javax.jbi.component.InstallationContext#getComponentClassName()
     */
    public String getComponentClassName()
    {
        return componentClassName;
    }
    
    /**
     * @see javax.jbi.component.InstallationContext#getComponentName()
     */
    public String getComponentName()
    {
        return componentName;
    }
    
    /**
     * @see javax.jbi.component.InstallationContext#getContext()
     */
    public ComponentContext getContext()
    {
        return context;
    }
    
    /**
     * @see javax.jbi.component.InstallationContext#getInstallationDescriptorExtension()
     */
    public DocumentFragment getInstallationDescriptorExtension()
    {
        return installationDescriptorExtension;
    }
    
    /**
     * @see javax.jbi.component.InstallationContext#getInstallRoot()
     */
    public String getInstallRoot()
    {
        return installRoot;
    }
    
    /**
     * @see javax.jbi.component.InstallationContext#isInstall()
     */
    public boolean isInstall()
    {
        return install;
    }
    
    /**
     * @see javax.jbi.component.InstallationContext#setClassPathElements(java.util.List)
     */
    public void setClassPathElements(List classPathElements)
    {
        this.classpathElements = classPathElements;
    }
    
}
