/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: ContainerConfig.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.config;

import java.util.HashSet;
import java.util.Iterator;

/**
 * Contains the whole configuration
 * of the container : 
 * - the container information (name, host),
 * - the JBI components (binding and engine) configuration
 * - the System services (directory, ...) configuration
 * @author Adrien LOUIS - EBM WebSourcing
 *
 */
public class ContainerConfig {
    
    private String host;
    
    private String identification;
    
    private int adminPort;
    
    private SystemServiceConfig directory;
    
    private SystemServiceConfig transporter;
    
    private SystemServiceConfig itineraryResolver;
    
    private SystemServiceConfig addressResolver;
    
    private HashSet componentsConfig = new HashSet();
    
    
    public void addComponentConfiguration(ComponentConfig c)
    {
        componentsConfig.add(c);
    }
    
    /**
     * Return the configurations of JBI components
     * @return an iterator of <i>ComponentConfig</i> object
     */
    public Iterator componentsConfigurations()
    {
        return componentsConfig.iterator();
    }
    
    /**
     * Return the name of the container
     * @return
     */
    public String getIdentification() 
    {
        return identification;
    }
    
    /**
     * Return the host name of the container
     * @return
     */
    public String getHost() 
    {
        return host;
    }
    
    public void setHost(String host) 
    {
        this.host = host;
    }
    
    public void setIdentification(String identification) 
    {
        this.identification = identification;
    }
    
    /**
     * Return the port for JMX administration throught RMI
     * @return
     */
    public int getAdminPort()
    {
        return adminPort;
    }
    
    public void setAdminPort(int adminPort)
    {
        this.adminPort = adminPort;
    }
    /**
     * Return the directory configuration
     * @return
     */
    public SystemServiceConfig getDirectory()
    {
        return directory;
    }
    
    public void setDirectory(SystemServiceConfig directory)
    {
        this.directory = directory;
    }
    
    /**
     * Return the transporter configuration
     * @return
     */
    public SystemServiceConfig getTransporter()
    {
        return transporter;
    }
    
    public void setTransporter(SystemServiceConfig transporter)
    {
        this.transporter = transporter;
    }
    
    /**
     * Return the addressResolver configuration
     * @return
     */
    public SystemServiceConfig getAddressResolver()
    {
        return addressResolver;
    }
    
    public void setAddressResolver(SystemServiceConfig addressResolver)
    {
        this.addressResolver = addressResolver;
    }
    
    /**
     * Return the itineraryResolver configuration
     * @return
     */
    public SystemServiceConfig getItineraryResolver()
    {
        return itineraryResolver;
    }
    
    public void setItineraryResolver(SystemServiceConfig itineraryResolver)
    {
        this.itineraryResolver = itineraryResolver;
    }
}
