/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: EndpointDirectory.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.directory;



import javax.jbi.JBIException;
import javax.naming.InitialContext;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.SystemServiceLifeCycle;
import org.objectweb.petals.routing.Address;



/**
 * Provide the component's endpoints referencing management in the directory.
 * @author Adrien LOUIS - EBM WebSourcing
 */
public interface EndpointDirectory extends SystemServiceLifeCycle{
    
    
    /**
     * Register the endpoint. 
     * <code>containerName</code>,<code>componentName</code> 
     * and <code>endpointName</code> attributes of
     * <code>Address</code> must be specified.
     * @param address The address representing the endpoint.
     * @throws JBIException directory accessing problem, or  address is invalid
     */
    void registerEndpoint(Address address)
    throws JBIException;
    
    /**
     * unregister the endpoint specified by the <code>Address</code>
     * parameter.
     * <code>containerName</code>,<code>componentName</code> 
     * and <code>endpointName</code> attributes of
     * <code>Address</code> must be specified.
     * @param address The address representing the endpoint.
     * @throws JBIException  directory accessing problem, or  address not found / is null
     */
    void unregisterEndpoint(Address address)
    throws JBIException;
    
    /**
     * Return an array of all addresses for the specified interface
     * @param interfaceName
     * @return a non null array
     * @throws JBIException
     */   
    Address[] getAddressesForInterface(QName interfaceName)
    throws JBIException;
    
    /**
     * Return an array of all addresses for the specified service
     * @param serviceName
     * @return a non null array 
     * @throws JBIException directory accessing problem or serviceName is null
     */   
    Address[] getAddressesForService(QName serviceName)
    throws JBIException;
    
    /**
     * Return an array of all addresses
     * @return a non null array
     * @throws JBIException directory accessing problem
     */   
    Address[] getAddresses()
    throws JBIException;
    
    /**
     * Return the address matching enpointName
     * @return the address of the endpoint. 
     * @throws JBIException directory accessing problem, endpointName is null or not found
     */   
    Address getAddressForEndpoint(String endpointName)
    throws JBIException;
    
    /**
     * Return the jndi initial context
     * @return
     * @throws JBIException
     */
    InitialContext getNamingContext()
    throws JBIException;
    
  
}