/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: DeliveryChannelImpl.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.messaging;



import javax.jbi.JBIException;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.context.ComponentContextImpl;
import org.objectweb.petals.routing.AddressFactory;
import org.objectweb.petals.routing.Router;
import org.objectweb.petals.util.ESBLogger;



/**
 * TODO DeliveryChannelImpl:check multi thread use
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class DeliveryChannelImpl implements DeliveryChannel
{
    protected ComponentContextImpl context;
    
    protected AddressFactory addressFactory ;

    protected Router router;
    
    protected IncomingMessageQueue queue;
    
    protected boolean opened = true;
	
    /**
     * Constructor
     * @param c
     */
    public DeliveryChannelImpl(ComponentContextImpl context, Router router, AddressFactory addressFactory)
    {
        ESBLogger.logFine(this,"<init>");
        
        queue = new IncomingMessageQueue();
        this.context = context;
        this.addressFactory = addressFactory;
        this.router = router;
    }
    
    /**
     * @see javax.jbi.messaging.DeliveryChannel#accept()
     */
    public MessageExchange accept() throws MessagingException
    {
        ESBLogger.logFine(this,"accept()");
        
        if(! isOpened())
        {
            throw new MessagingException("DeliveryChannel is closed.");
        }
        return queue.poll();
    }
    
    
    /**
     * @see javax.jbi.messaging.DeliveryChannel#accept(long)
     * TODO accept:synchronize
     */
    public MessageExchange accept(long timeoutMS) throws MessagingException
    {
        ESBLogger.logFine(this,"accept("+timeoutMS+")");
        
        checkDeliveryChannelIsOpened();
        
        MessageExchangeImpl msg;
        try
        {
            msg = queue.poll(timeoutMS);
        }
        catch (Exception e)
        {
            throw new MessagingException(e);
        }
        return msg;
    }
    
    /**
     * (non-Javadoc)
     * @see javax.jbi.messaging.DeliveryChannel#close()
     * TODO close:manage the received messages which are not consummed and are still in the incoming message queue.
     */
    public synchronized void close() throws MessagingException
    {
        ESBLogger.logFine(this,"close()");
        
        opened = false;
    }
        
    /**
     * @see javax.jbi.messaging.DeliveryChannel#createExchangeFactory()
     */
    public MessageExchangeFactory createExchangeFactory()
    {
        ESBLogger.logFine(this,"createExchangeFactory()");
        
        return new MessageExchangeFactoryImpl(addressFactory,context,this);
    }
    
    /**
     * @see javax.jbi.messaging.DeliveryChannel#createExchangeFactory(javax.xml.namespace.QName)
     */
    public MessageExchangeFactory createExchangeFactory(QName interfaceName)
    {
        ESBLogger.logFine(this,"createExchangeFactory("+interfaceName+")");
        
        MessageExchangeFactoryImpl mef = new MessageExchangeFactoryImpl(addressFactory,context,this);
        
        mef.setDefaultInterfaceName(interfaceName);
        
        return mef;
    }
    
    /**
     * @see javax.jbi.messaging.DeliveryChannel#createExchangeFactory(javax.jbi.servicedesc.ServiceEndpoint)
     */
    public MessageExchangeFactory createExchangeFactory(ServiceEndpoint endpoint)
    {
        ESBLogger.logFine(this,"createExchangeFactory("+endpoint+")");
        
        MessageExchangeFactoryImpl mef = new MessageExchangeFactoryImpl(addressFactory,context,this);
        
        mef.setDefaultServiceEndpoint(endpoint);
        
        return mef;
    }
    
    /**
     * @see javax.jbi.messaging.DeliveryChannel#createExchangeFactoryForService(javax.xml.namespace.QName)
     */
    public MessageExchangeFactory createExchangeFactoryForService(
            QName serviceName)
    {
        ESBLogger.logFine(this,"createExchangeFactory("+serviceName+")");
        
        MessageExchangeFactoryImpl mef = new MessageExchangeFactoryImpl(addressFactory,context,this);
        
        mef.setDefaultServiceName(serviceName);
        
        return mef;
    }
    
    /**
     * @see javax.jbi.messaging.DeliveryChannel#send(javax.jbi.messaging.MessageExchange)
     */
    public void send(MessageExchange exchange) throws MessagingException
    {
        ESBLogger.logFine(this,"send("+exchange+")");
        
        checkDeliveryChannelIsOpened();
        checkMessageExchange(exchange);
        
        MessageExchangeImpl exchangeImpl = (MessageExchangeImpl)exchange;
        
        try 
        {
            router.send(exchangeImpl,context);
        }
        catch (JBIException e) 
        {
            throw new MessagingException(e);
        }
    } 

    
    /**
     * @see javax.jbi.messaging.DeliveryChannel#sendSynch(javax.jbi.messaging.MessageExchange)
     */
    public boolean sendSync(MessageExchange exchange)
    throws MessagingException
    {
        checkDeliveryChannelIsOpened();
        checkMessageExchange(exchange);
        throw new NoSuchMethodError();
    }
    
    /**
     * @see javax.jbi.messaging.DeliveryChannel#sendSynch(javax.jbi.messaging.MessageExchange, long)
     */
    public boolean sendSync(MessageExchange exchange, long timeoutMS)
    throws MessagingException
    {
        checkDeliveryChannelIsOpened();
        checkMessageExchange(exchange);
        throw new NoSuchMethodError();
    }
    
    /////////////
    // non jbi
    ////////////
    
    public synchronized boolean isOpened()
    {
        return opened;
    }
    
    /**
     * Check if the <code>DeliveryChannel</code> is opened
     * @throws MessagingException
     */
    public synchronized void checkDeliveryChannelIsOpened() throws MessagingException
    {
        if(! opened)
        {
            throw new MessagingException("DeliveryChannel is closed.");
        }
    }
    
    /**
     * Check if the <code>MessageExchange</code> is Ok
     * TODO checkMessageExchange:only test if messageExchange is null
     * @throws MessagingException
     */
    public void checkMessageExchange(MessageExchange exchange) throws MessagingException
    {
        if(exchange == null)
        {
            throw new MessagingException("MessageExchange is null.");
        }
    }
    
    /**
     * Used by the <code>IncomingMessageDispatcher</code> when a
     * message is received by the <code>Transporter</code> and the
     * recepient of the message is the component attached to this channel.
     * The message is put in the channel queue, and will be consummed
     * when the component calls an <i>accept</i> mehod.
     * @param msg the message received by the <code>Transporter</code>
     * @throws JBIException the channel has been closed by the component. The message can not
     * be accepted
     */
    public void push(MessageExchangeImpl msg) throws JBIException
    {
        checkDeliveryChannelIsOpened();
        
     	queue.push(msg);
    }
}
