/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: MessageExchangeImpl.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.messaging;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Set;

import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;

import javax.xml.namespace.QName;

import org.objectweb.petals.routing.Itinerary;
import org.objectweb.petals.util.ESBLogger;


/**
 * @author Adrien LOUIS - EBM WebSourcing
 *
 */
public class MessageExchangeImpl implements MessageExchange 
{
    
    protected ServiceEndpoint endpoint;
    
    protected QName interfaceName;
    
    protected QName service;
    
    protected Exception error;
    
    protected Fault fault;
   
    protected HashMap messages = new HashMap();
    
    protected QName operation;
    
    protected URI pattern;
    
    protected HashMap properties = new HashMap();
    
    protected String exchangeId;
    
    protected Role role;
    
    protected ExchangeStatus status;
    
    protected boolean transacted;
    
    protected Itinerary itinerary;
    
    ////////////////////////////////////////////////////////////
    //
    // implementation specific attributes
    //
    ////////////////////////////////////////////////////////////
    
    protected boolean request = true;
    
    public static final String IN_MSG = "in";
    public static final String OUT_MSG = "out";
    
    public static URI IN_ONLY_PATTERN;
    public static URI IN_OPTIONAL_OUT_PATTERN;
    public static URI IN_OUT_PATTERN;
    public static URI ROBUST_IN_ONLY_PATTERN;
    
    static
    {
        try 
        {
            IN_ONLY_PATTERN = new URI("http://www.w3.org.2004/08/wsdl/in-only");
            
            IN_OPTIONAL_OUT_PATTERN = new URI("http://www.w3.org.2004/08/wsdl/in-opt-out");
            
            IN_OUT_PATTERN = new URI("http://www.w3.org.2004/08/wsdl/in-out");
            
            ROBUST_IN_ONLY_PATTERN = new URI("http://www.w3.org.2004/08/wsdl/robust-in-only");
        } 
        catch (URISyntaxException e) 
        {
            ESBLogger.logSevere("unexpected Exception in URI creation",e);
            // Should never be thrown
        }
    }
    
    /**
     * Default constructor
     *
     */
    public MessageExchangeImpl()
    {	
        itinerary = new Itinerary();
        
        status = ExchangeStatus.ACTIVE;
        
        role = Role.CONSUMER;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#createFault()
     */
    public Fault createFault() throws MessagingException 
    {
        return new FaultImpl();
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#createMessage()
     */
    public NormalizedMessage createMessage() throws MessagingException 
    {
        return new NormalizedMessageImpl();
    }
    
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getEndpoint()
     */
    public ServiceEndpoint getEndpoint() 
    {
        return endpoint;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getError()
     */
    public Exception getError() 
    {
        return error;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getExchangeId()
     */
    public String getExchangeId() 
    {
        return exchangeId;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getFault()
     */
    public Fault getFault() 
    {
        return fault;
    }
    
    public QName getInterfaceName()
    {
        return interfaceName;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getMessage(java.lang.String)
     */
    public NormalizedMessage getMessage(String name) 
    {
        return (NormalizedMessage)messages.get(name);
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getOperation()
     */
    public QName getOperation() 
    {
        return operation;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getPattern()
     */
    public URI getPattern() 
    {
        return pattern;
    }
    
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getProperty(java.lang.String)
     */
    public Object getProperty(String name) 
    {
        return properties.get(name);
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getService()
     */
    public QName getService() 
    {
        return service;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getPropertyNames()
     */
    public Set getPropertyNames()
    {
        return properties.keySet();
    }
    
    /**
     * (suivant que l on cr un nouveau ou qu'on rcupre de la couche de transport)
     * (non-Javadoc)
     * @see javax.jbi.messaging.MessageExchange#getRole()
     */
    public Role getRole()
    {
        return role;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#getStatus()
     */
    public javax.jbi.messaging.ExchangeStatus getStatus() 
    {
        return status;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#isTransacted()
     */
    public boolean isTransacted() 
    {
        return transacted;
    }
      
    /**
     * @see javax.jbi.messaging.MessageExchange#setEndpoint(java.lang.Exception)
     */
    public void setEndpoint(ServiceEndpoint endpoint) 
    {
        this.endpoint = endpoint;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#setError(java.lang.Exception)
     */
    public void setError(Exception error) 
    {
        this.error = error;
        
        this.status = ExchangeStatus.ERROR;
    }
    
    /**
     * (non-Javadoc)
     * @see javax.jbi.messaging.MessageExchange#setFault(javax.jbi.messaging.Fault)
     */
    public void setFault(Fault fault) throws MessagingException 
    {
        this.fault = fault;
    }
    
    public void setInterfaceName(QName interfaceName)
    {
        this.interfaceName = interfaceName;
    }
    
    /**
     * TODO setMessage:manage associated MEP ?
     * (non-Javadoc)
     * @see javax.jbi.messaging.MessageExchange#setMessage(javax.jbi.messaging.NormalizedMessage, java.lang.String)
     */
    public void setMessage(NormalizedMessage msg, String name)
    throws MessagingException 
    {
        messages.put(name,msg);
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#setOperation(javax.xml.namespace.QName)
     */
    public void setOperation(QName name) 
    {
        this.operation = name;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#setProperty(java.lang.String, java.lang.Object)
     */
    public void setProperty(String name, Object obj) 
    {
        properties.put(name,obj);
    }
    
    /**
     * set Destination ServiceName.
     * A destinsation Endpointreference is created with only "serviceName" attribute
     */
    public void setService(QName service) 
    {
        this.service = service;
    }
    
    /**
     * @see javax.jbi.messaging.MessageExchange#setStatus(javax.jbi.messaging.ExchangeStatus)
     */
    public void setStatus(ExchangeStatus status) throws MessagingException 
    {
        this.status = status;
    }
    
    ///////////////////////
    // non jbi
    /////////////////////
    
    public Itinerary getItinerary()
    {
        return itinerary;
    }
    
    
    public void setItinerary(Itinerary itinerary)
    {
        this.itinerary = itinerary;
    }
    /**
     * @param exchangeId The exchangeId to set.
     */
    public void setExchangeId(String exchangeId) {
        this.exchangeId = exchangeId;
    }
    /**
     * @param pattern The pattern to set.
     */
    public void setPattern(URI pattern) {
        this.pattern = pattern;
    }
    /**
     * @param transacted The transacted to set.
     */
    public void setTransacted(boolean transacted) {
        this.transacted = transacted;
    }
    
    
    
    public void setRole(Role role)
    {
        this.role = role;
    }
    public boolean isRequest()
    {
        return request;
    }
    
    public void setRequest(boolean request)
    {
        this.request = request;
    }
    public boolean isResponse()
    {
        return ! request;
    }
    
    public String toString()
    {
        return "MessageExchange@"+getExchangeId();
    }
    
    
}
