/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: NormalizedMessageImpl.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.messaging;

import java.util.HashMap;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.security.auth.Subject;
import javax.xml.transform.Source;

/**
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class NormalizedMessageImpl implements NormalizedMessage {

	private HashMap attachments;
	
	private Source content;
	
	private HashMap properties;
	
	private Subject securitySubject;
	
	public NormalizedMessageImpl()
	{
		attachments = new HashMap();
		
		properties = new HashMap();
	}
	
	/**
	 * @see javax.jbi.messaging.NormalizedMessage#addAttachment(java.lang.String, javax.activation.DataHandler)
	 */
	public void addAttachment(String id, DataHandler content)
			throws MessagingException 
	{
		attachments.put(id,content);
	}

	/**
	 * @see javax.jbi.messaging.NormalizedMessage#getAttachment(java.lang.String)
	 */
	public DataHandler getAttachment(String id) 
	{
		return (DataHandler)attachments.get(id);
	}

	/**
	 * @see javax.jbi.messaging.NormalizedMessage#getContent()
	 */
	public Source getContent() 
	{
		return content;
	}

	/**
	 * @see javax.jbi.messaging.NormalizedMessage#getProperty(java.lang.String)
	 */
	public Object getProperty(String name) 
	{
		return properties.get(name);
	}

	/**
	 * @see javax.jbi.messaging.NormalizedMessage#getPropertyNames()
	 */
	public Set getPropertyNames() 
	{
		return properties.keySet();
	}

	/**
	 * @see javax.jbi.messaging.NormalizedMessage#getSecuritySubject()
	 */
	public Subject getSecuritySubject() 
	{
		return securitySubject;
	}

	/**
	 * @see javax.jbi.messaging.NormalizedMessage#removeAttachment(java.lang.String)
	 */
	public void removeAttachment(String id) throws MessagingException 
	{
		Object att = attachments.remove(id);
		
		if(att == null)
		{
		    throw new MessagingException(id+" attachment does not exist.");
		}
	}

	/**
	 * @see javax.jbi.messaging.NormalizedMessage#setContent(javax.xml.transform.Source)
	 */
	public void setContent(Source content) throws MessagingException 
	{
		this.content = content;
	}

	/**
	 * @see javax.jbi.messaging.NormalizedMessage#setProperty(java.lang.String, java.lang.Object)
	 */
	public void setProperty(String name, Object value) 
	{
		properties.put(name,value);
	}

	/**
	 * @see javax.jbi.messaging.NormalizedMessage#setSecuritySubject(javax.security.auth.Subject)
	 */
	public void setSecuritySubject(Subject subject) 
	{
		this.securitySubject = subject;
	}

	/**
	 * @see javax.jbi.messaging.NormalizedMessage#getAttachmentNames()
	 */
    public Set getAttachmentNames()
    {
        return attachments.keySet();
    }
}
