/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: AddressFactory.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.routing;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.context.ComponentContextImpl;
import org.objectweb.petals.util.ESBLogger;


/**
 * Create <code>Address</code> elements for <code>Itinerary</code>.
 * An address can be initialized with local information 
 * (component and container information of the component which owns the
 *  related <code>DeliveryChannel</code>)
 * or with <code>MessageExchange</code> properties set for an exchange.
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class AddressFactory
{
    String containerName;
    
    public AddressFactory(String containerName)
    {
        this.containerName = containerName;
    }
    
    /**
     * create an <code>Address</code> with <code>interfaceName</code>,
     * <code>service</code> and <code>endpointName</code> properties.
     * if <code>serviceName</code> and
     * <code>ServiceEndpoint</code> 'service properties exist in
     * <code>MessageExchange</code>, <code>ServiceEndpoint</code> 'service 
     * is prior.
     * @param msg
     * @return
     */
    public Address createAddressFromMessageExchange(MessageExchange msg)
    {
        ESBLogger.logFine(this,"createAddressFromMessageExchange("+msg+")");
        
        QName interfaceName = msg.getInterfaceName();
        
        QName serviceName = msg.getService();
        
        String endpoint = null;
        
        if(msg.getEndpoint() != null)
        {
            if(msg.getEndpoint().getServiceName() != null)
            {
                serviceName = msg.getEndpoint().getServiceName();
                
                endpoint = msg.getEndpoint().getEndpointName();
            }
        }
        return new Address(interfaceName,serviceName,endpoint,null,null);
    }
    
    /**
     * create an <code>Address</code> with <code>componentName</code>
     * and <code>containerName</code> related to the context of
     * the component that initiated the exchange.
     * @param source
     * @return
     */
    public Address createAddressForLocalSource(ComponentContextImpl source)
    {
        ESBLogger.logFine(this,"createAddressForLocalSource("+source+")");
        
        String componentName = source.getComponentName();
        
        return new Address(null,null,null,componentName,containerName);
    }
    
    /**
     * create an <code>Address</code> with <code>componentName</code>
     * and <code>containerName</code> related to the context of
     * the component that initiated the exchange.
     * <code>ServiceEndpoint</code> information are set.
     * TODO createAddressForLocalServiceEndpoint: manage interfaces[]
     * @param endpoint
     * @param source
     * @return an address with endpoint, component and container information
     */
    public Address createAddressForLocalServiceEndpoint(ServiceEndpoint endpoint, ComponentContextImpl source)
    {
        ESBLogger.logFine(this,"createAddressForLocalServiceEndpoint("+endpoint+","+source+")");
        
        String componentName = source.getComponentName();
        
        return new Address(null,endpoint.getServiceName(),endpoint.getEndpointName(),componentName,containerName);
    }
    
    /**
     * Return an unique ID for the creation of Message.
     * ID is based on the name of the container and the
     * System.currentTimeMillis value.
     * @return an unique ID
     */
    public synchronized String createId()
    {
        return containerName+"_"+System.currentTimeMillis();
    }
}
