/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: AddressResolver.java,v 1.3 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.routing;

import org.objectweb.petals.component.SystemServiceLifeCycle;
import org.objectweb.petals.component.context.ComponentContextImpl;
import org.objectweb.petals.description.ContainerDescription;
import org.objectweb.petals.directory.Directory;
import org.objectweb.petals.messaging.MessageExchangeImpl;


/**
 * This component is used by the container to resolve addresses
 * contained in a message itinerary.
 * An address can be an interface name, a service name, or an
 * endpoint name.
 * This component find the endpoint satisfying the address,
 * by using defined rules.
 * 
 * Then this component has to fill the address with the containerName 
 * and the componentName that are hosting the endpoint.
 * 
 * This is the interface that have to be implemented as 
 * the container's systemService AddressResolver component.
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 */
public interface AddressResolver extends SystemServiceLifeCycle
{
    /**
     * Resolve the address if necessary.
     * Resolution consists in finding an endpoint satisfying
     * <code>interface</code> or <code>service</code>.
     * <br>The source of the exchange and the exchange help
     * the <code>AddressResolver</code> to find the endpoint.
     * <br>The <code>containerName</code> and <code>ComponentName</code>
     * properties are set.
     * <br>If <code>containerName</code> and <code>ComponentName</code> 
     * already exist, do nothing.
     * @param address
     * @param source
     * @param exchange
     */
    void resolveAddress(Address address,
            ComponentContextImpl source, MessageExchangeImpl exchange)
            throws RoutingException;

    /**
     * Specify to <code>AddressResolver</code> the <code>ContainerDescription</code>
     * @param container
     */
    void initContainerDescription(ContainerDescription container);
   
    /**
     * Specify to <code>AddressResolver</code> the <code>Directory</code> 
     * @param directory
     */
    void initDirectory(Directory directory);
}