/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: Transporter.java,v 1.3 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.transport;


import org.objectweb.petals.component.SystemServiceLifeCycle;
import org.objectweb.petals.description.ContainerDescription;
import org.objectweb.petals.messaging.MessageExchangeImpl;
import org.objectweb.petals.routing.Address;
import org.objectweb.petals.transport.NormalizedMessageListener;



/**
 * Transporter Plug-in interface for message receiver/sender .
 * The framework set to the transporter plug-in a <i>NormalizedMessageListener</i>
 * in which the Transporter Plug-in place received messages.
 * 
 * For sending or receiving messages,
 * the transporter has to use
 * <code>org.objectweb.petals.description.ServiceContainerReference</code>
 * and <code>org.objectweb.petals.description.ComponentReference</code>
 * referenced in <code>javax.jbi.messaging.MessageExchange</code> properties list
 * as <i>org.objectweb.petals.messaging.MessageExchange.SENDER_CONTAINER</i>
 * as <i>org.objectweb.petals.messaging.MessageExchange.SENDER_COMPONENT</i>
 * as <i>org.objectweb.petals.messaging.MessageExchange.RECEIVER_CONTAINER</i>
 * as <i>org.objectweb.petals.messaging.MessageExchange.RECEIVER_COMPONENT</i>
 * 
 * This is the interface that have to be implemented as 
 * the container's systemService Transporter component.
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 */
public interface Transporter extends SystemServiceLifeCycle{

	/**
	 * Called by the framework at plug-in initialization,
	 * in order to give it the message listener where received messages will be redirected.
	 * @param queue
	 */
	void setNormalizedMessageListener(NormalizedMessageListener queue);
	
	void setContainerDescription(ContainerDescription serviceContainerReference);
	     
	void send(MessageExchangeImpl messageExchange, Address destination) throws TransportException;
	
}
