/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: MBeanHelper.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.util;


import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;

import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXConnectorServerFactory;
import javax.management.remote.JMXServiceURL;

import java.rmi.registry.LocateRegistry;

/**
 * Class for initialize and accessing the JMX Server.
 * 
 * @author Adrien LOUIS - EBM WebSourcing
  */
public class MBeanHelper
{
    private MBeanServer mbs;
    
    private JMXConnectorServer cs;
    
    private ObjectName cntorServerName;
    
    public MBeanHelper(int adminPort)
    {
        ESBLogger.logFine(this,"<init>("+adminPort+")");
        
        mbs = MBeanServerFactory.createMBeanServer();
        
        try
        {
            // start the rmi registry
            LocateRegistry.createRegistry(adminPort);
            
            JMXServiceURL jmxUrl = new JMXServiceURL("service:jmx:rmi:///jndi/rmi://localhost:"+adminPort+"/server");
            
            cntorServerName = ObjectName.getInstance("connectors:protocol=rmi");
            
            cs = JMXConnectorServerFactory.newJMXConnectorServer(jmxUrl, null, mbs);
            
            mbs.registerMBean(cs, cntorServerName);
            
            cs.start();
            
            ESBLogger.logFinest(this,"JMX Server started on RMI port "+adminPort);
            ESBLogger.logFinest(this,"service:jmx:rmi:///jndi/rmi://localhost:"+adminPort+"/server");
        }
        catch (Exception e)
        {
            ESBLogger.logSevere(this,"<init>", e);
        }
        
    }
    
    public MBeanServer getMBeanServer()
    {
        return mbs;
    }

    
    
}
