/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: DirectoryImpl.java,v 1.1 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package com.ebmwebsourcing.jbi.systemservicesimpl;


import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;

import javax.jbi.JBIException;
import javax.naming.Binding;
import javax.naming.Context;
import javax.naming.InitialContext;

import javax.naming.Name;
import javax.naming.NameAlreadyBoundException;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.NotContextException;
import javax.xml.namespace.QName;

import org.objectweb.petals.description.ContainerDescription;
import org.objectweb.petals.description.ServiceEndpointImpl;
import org.objectweb.petals.directory.Directory;
import org.objectweb.petals.routing.Address;
import org.objectweb.petals.util.ESBLogger;
import org.objectweb.petals.util.NamingHelper;
import org.objectweb.petals.util.StringHelper;


/**
 * @author Adrien LOUIS, Maxime CARPENTIER - EBM WebSourcing
 *
 * TODO DirectoryImpl:manage services "interfaces" in the directory
 */
public class DirectoryImpl implements Directory{
    
    private InitialContext rootContext;
    
    private Context containersContext;
    
    private Context endpointsContext;
    
    private Context servicesContext;
    
    
    public static final String CONTAINERS_REF="containers";
    
    public static final String ENDPOINTS_REF="endpoints";
    
    public static final String SERVICES_REF="services";
    
    
    /**
     * register the container:
     * containers
     * 		+ ("containerName",ContainerDescription)
     * @param ed
     * @throws JBIException 
     */
    public void registerContainer(ContainerDescription cr) throws JBIException
    {
        ESBLogger.logFine(this,"registerContainer("+cr+")");
        
        if ((cr != null) && (cr.getHost() == null)) { throw new JBIException("host is null"); }
        try 
        {
            containersContext.bind(cr.getName(), cr);
            
        } 
        catch (Exception e) 
        {
            ESBLogger.logSevere(this,"registerContainer()", e);
            
            throw new JBIException(e);
        }
    }
    
    /**
     * unregister the container
     * @param cr
     * @throws JBIException
     */
    public void unregisterContainer(ContainerDescription cr) throws JBIException
    {
        ESBLogger.logFine(this,"unregisterContainer("+cr+")");
        
        try {
            containersContext.lookup(cr.getName());
        } catch (Exception e1) {
            ESBLogger.logSevere(this,"unregisterContainer()", e1);
            throw new JBIException(e1);
        } 
        try 
        {
            containersContext.unbind(cr.getName());
        } 
        catch (NamingException e) 
        {
            ESBLogger.logSevere(this,"unregisterContainer()", e);
            
            throw new JBIException(e);
        }
    }
    
    
    
    /**
     * register an endpoint in the service directory
     * bind its adress in the services and endpoints context
     * services
     * 		- <serviceName>
     *   		+ (endpointName,Address)
     * endpoints
     * 		+ (endpointName,Address)
     * @param address of the endpoint
     */
    public void registerEndpoint(Address address) throws JBIException
    {
        ESBLogger.logFine(this,"registerEndpoint("+address+")");
        
        if(address == null || address.getComponent() == null || address.getContainer() == null || address.getEndpoint() == null)
        {
            throw new JBIException("Can not register endpoint. Address is invalid : "+address);
        }
        try {
            // bind in endpoints context
            endpointsContext.bind(address.getEndpoint(), address);
            
            // bind in services context 
            // if no serviceName is specified, do nothing
            
            String s_serviceName = NamingHelper.QNameToString(address.getService());
            if(StringHelper.isEmpty(s_serviceName)) { return; }
            Name serviceName = NamingHelper.StringToName(s_serviceName);
            if (!isService(serviceName)) { 
                servicesContext.createSubcontext(serviceName);
            }
            servicesContext.bind(serviceName.add(address.getEndpoint()), address);
        } catch (NamingException e) {
            ESBLogger.logSevere(this,"registerEndpoint()", e);
            throw new JBIException(e);
        }
    }
    
    
    
    /**
     * unregister endpoint in directory.</br>
     * unbind endpoint from endpoints and services contexts
     * @see org.objectweb.petals.directory.EndpointDirectory#unregisterEndpoint(org.objectweb.petals.routing.Address)
     */
    public void unregisterEndpoint(Address address) throws JBIException
    {
        ESBLogger.logFine(this,"unregisterEndpoint("+address+")");
        
        try
        {
            // unbind from endpoints context
            endpointsContext.unbind(address.getEndpoint());
            
            // unbind from services context 
            // if the endpoint serve a service
            String s_serviceName = NamingHelper.QNameToString(address.getService());
            if(! StringHelper.isEmpty(s_serviceName))
            {
                Name serviceName = NamingHelper.StringToName(s_serviceName);
                servicesContext.unbind(serviceName.add(address.getEndpoint()));
                // destroy service subcontext if no more endpoints serve this services
                serviceName = NamingHelper.QNameToName(address.getService());
                if(! servicesContext.list(serviceName).hasMore())
                {
                    servicesContext.destroySubcontext(serviceName);
                }
            }
        }
        catch (Exception e)
        {
            ESBLogger.logSevere(this,"unregisterEndpoint()",e);
            throw new JBIException(e);
        }
    }
    
    /**
     * @see org.objectweb.petals.directory.EndpointDirectory#getAddressesForInterface(javax.xml.namespace.QName)
     * 	TODO getAddressesForInterface
     */
    public Address[] getAddressesForInterface(QName interfaceName) throws JBIException
    {
         throw new NoSuchMethodError();
    }
    
    /**
     * Return all endpoints addresses
     * @see org.objectweb.petals.directory.EndpointDirectory#getAddresses()
     * @return all addresses binded in the endpoints context
     */
    
    public Address[] getAddresses() throws JBIException
    {
        ESBLogger.logFine(this,"getAddresses()");
        
        Address[] result = null;
        try 
        {
            ArrayList l = new ArrayList();
            //get all endpoint references for the service
            for(NamingEnumeration e = rootContext.listBindings(ENDPOINTS_REF);e.hasMore();)
            {
                Binding name = (Binding)e.next();
                //get address and add it to the list  
                l.add((Address) name.getObject());
            }
            result = (Address[])l.toArray(new Address[0]);
        } 
        catch (NamingException e) 
        {
            // should not happen
            ESBLogger.logSevere(this,"getAddresses()",e);
            throw new RuntimeException(e);
        }
        return result;
        
    }
    
    /**
     * Return true if service is binded in the services context
     * @param serviceName
     * @return true if service is binded in the services context
     * @throws JBIException
     */
    private boolean isService(Name serviceName) throws JBIException
    {
        boolean result = false;
        
        try 
        {
            ArrayList l = new ArrayList();
            String name = "" ;
        
            //get all services references for the service
            for(NamingEnumeration e = rootContext.listBindings(SERVICES_REF);e.hasMore();)
            {
                name  = ((Binding)e.next()).getName();
                if (name.equalsIgnoreCase(serviceName.toString())) { return true; }
            }
            
        } 
        catch (NamingException e) 
        {
            ESBLogger.logSevere(this,"getAddresses()",e);
            throw new RuntimeException(e);
        }
        return result;
        
    }
    
    
    /**
     * Return an array of all addresses for a service
     * @param serviceName
     * @return a non null array
     */   
    public Address[] getAddressesForService(QName serviceName) throws JBIException
    {
        ESBLogger.logFine(this,"getAddressesForService("+serviceName+")");
        
        if (serviceName == null) 
        { 
            throw new JBIException("null serviceName"); 
        }
        Name n = NamingHelper.QNameToName(serviceName);
        
        // check if the service is referenced in the directory
        try
        {
            servicesContext.lookup(n);
        }
        catch (NamingException e1)
        {
            // service is not referenced, return an empty array
            return new Address[0];
        }
        Address[] result = null;
        
        try 
        {
            ArrayList l = new ArrayList();
            
            //get all endpoint references for the service
            for(NamingEnumeration e = servicesContext.listBindings(n);e.hasMore();)
            {
                Binding name = (Binding)e.next();
                
                //get address and add it to the list  
                l.add((Address) name.getObject());
            }
            result = (Address[])l.toArray(new Address[0]);
        } 
        catch (NamingException e) 
        {
            // should not happen
            ESBLogger.logSevere(this,"getAddressesForService("+serviceName+")",e);
            throw new RuntimeException(e);
        }
        return result;
        
    }
    
    
    /**
     * @see org.objectweb.petals.directory.EndpointDirectory#getAddressForEndpoint(java.lang.String)
     * 
     */
    public Address getAddressForEndpoint(String endpointName) throws JBIException
    {
        ESBLogger.logFine(this,"getAddressForEndpoint("+endpointName+")");
        
        Address result;
        try 
        {
            result = (Address) endpointsContext.lookup(endpointName);
        } 
        catch (NamingException e) 
        {
            // should not happen
            ESBLogger.logSevere(this,"getAddressForEndpoint("+endpointName+")",e);
            throw new JBIException(e);
        }
        return result;
    }
    
    
    
    /**
     * @param interfaceName
     * @return
     * TODO getEndpoints
     */
    public ServiceEndpointImpl[] getEndpoints(QName interfaceName) 
    {
        throw new NoSuchMethodError();
    }
    
    
    
    /**
     * find a <code>ContainerReference</code> by its name
     * @param containerName
     * @return null if no matching container name were found
     * @throws directory accessing problem
     */   
    public ContainerDescription findContainerByName(String containerName) throws JBIException
    {
        ESBLogger.logFine(this,"findContainerByName("+containerName+")");
        
        ContainerDescription result = null;
        
        if (containerName == null) 
        { 
            throw new JBIException("containerName null"); 
        }
        try 
        {
            ArrayList l = new ArrayList();
            String name = "" ;
            //get all services references for the service
            for(NamingEnumeration e = rootContext.listBindings(CONTAINERS_REF);e.hasMore();)
            {
                Binding tmp = (Binding)e.next();
                name  = tmp.getName();
                if (name.equalsIgnoreCase(containerName.toString())) { return (ContainerDescription)tmp.getObject(); }
            }
            
        } 
        catch (NamingException e) 
        {
            ESBLogger.logSevere(this,"findContainerByName()",e);
            throw new RuntimeException(e);
        }
        return result;
        
    }
    
    
    /**
     * Create and / or reference contexts "containers", "endpoints" and "services"
     * in the global directory.
     * TODO initContexts:manage creation-or-lookup with another mecanism than NameAlreadyBoundException
     * @throws JBIException problems while creating or lookup contexts
     */
    public void initContexts() throws JBIException
    {
        ESBLogger.logFine(this,"initContexts()");
        
        try 
        {
            URL url = ESBLogger.class.getResource("/jndi.properties");
            Properties p = new Properties();
            p.load(url.openStream());
            
            rootContext = new InitialContext(p);
            
            try{
                containersContext = rootContext.createSubcontext(CONTAINERS_REF);
            }catch(NameAlreadyBoundException e){
                containersContext = (Context) rootContext.lookup(CONTAINERS_REF);
            }
            try {
                endpointsContext = rootContext.createSubcontext(ENDPOINTS_REF);
            }catch(NameAlreadyBoundException e){
                endpointsContext = (Context) rootContext.lookup(ENDPOINTS_REF);
            }
            try {
                servicesContext = rootContext.createSubcontext(SERVICES_REF);
            }catch(NameAlreadyBoundException e){
                servicesContext = (Context) rootContext.lookup(SERVICES_REF);
            }
            if(containersContext == null || endpointsContext == null || servicesContext == null)
            {
                throw new NotContextException();
            }
        } 
        catch (Exception e) 
        {
            ESBLogger.logSevere(this,"initContexts()", e);
            throw new JBIException(e);
        }
    }
    
    public InitialContext getNamingContext()
    {
        return rootContext;
    }
    
    /////////////////////////////////////////////////////////////////
    //
    // Life Cycle Implementation
    //
    /////////////////////////////////////////////////////////////////
    
    
    /**
     * Start local JNDI service if not already started
     * Create and / or reference contexts "containers", "endpoints" and "services"
     * @throws JBIException problems while creating or lookup contexts
     */
    public void init(HashMap context) throws JBIException
    {
        ESBLogger.logFine(this,"init("+context+")");
        
        try 
        {
           JoramLifeCycle.getInstance().startServer();
           initContexts();
        } 
        catch (Exception e) 
        {
            ESBLogger.logSevere(this,"init()", e);
            throw new JBIException(e);
        }
    }
    
    
    public void start() throws JBIException
    {
        //nothing
    }
    
    
    
    public void stop() throws JBIException
    {
        // nothing
    }
    
    /**
     * Stop local JNDI service
     */
    public void shutDown() throws JBIException
    {
        ESBLogger.logFine(this,"shutDown()");
        
        JoramLifeCycle.getInstance().stopServer();
    }
    
 
}
