/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: JoramLifeCycle.java,v 1.1 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package com.ebmwebsourcing.jbi.systemservicesimpl;

import java.net.URL;
import java.util.Properties;

import org.objectweb.petals.util.ESBLogger;


/**
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class JoramLifeCycle {
    
    private static JoramLifeCycle instance = new JoramLifeCycle();
    
    private JoramLifeCycle()
    {
    }
    
    public static JoramLifeCycle getInstance()
    {
        return instance;
    }
    
    /**
     * Start the local Joram server if not already started
     * @throws Exception
     */
    public synchronized void startServer() throws Exception
    {
        ESBLogger.logFine(this,"startDirectoryService()");
        if(! isServerRunning())
        {
            int id=-1;
            
            try 
            {
                // find server Id with "joram.properties" file
                
                URL url = ESBLogger.class.getResource("/joram.properties");
                
                Properties p = new Properties();
                
                p.load(url.openStream());
                
                id = Integer.parseInt(p.getProperty("id"));
                
                fr.dyade.aaa.agent.AgentServer.init((short) id, "./"+id, null);
                
                fr.dyade.aaa.agent.AgentServer.start();
                
                ESBLogger.logFinest(this,"startDirectoryService() : server '"+id+"' started, jndi server started");
            } 
            catch (Exception e) 
            {
                ESBLogger.logSevere(this,"startDirectoryService",e);
                
                fr.dyade.aaa.agent.AgentServer.stop(true);
                
                ESBLogger.logFinest(this,"startDirectoryService() :error, server '"+id+"' stopped");
                
                throw e;
            }
        }
    }
    
    /**
     * Stop the local Joram server
     *
     */
    public synchronized void stopServer()
    {
        ESBLogger.logFine(this,"stopDirectoryService()");
        
        fr.dyade.aaa.agent.AgentServer.stop();	
        
        ESBLogger.logFinest(this,"stopDirectoryService() : server stopped");
    }
    
    /**
     * Is the local joram server running or not
     * @return
     */    
    private boolean isServerRunning()
    {
        return (4==fr.dyade.aaa.agent.AgentServer.getStatus());	
    }
}
