/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: TransporterImpl.java,v 1.1 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package com.ebmwebsourcing.jbi.systemservicesimpl;



import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;

import javax.jbi.JBIException;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueReceiver;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.TextMessage;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.xml.namespace.QName;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;


import org.objectweb.joram.client.jms.Queue;
import org.objectweb.joram.client.jms.admin.AdminModule;
import org.objectweb.joram.client.jms.admin.User;
import org.objectweb.joram.client.jms.tcp.QueueTcpConnectionFactory;
import org.objectweb.joram.client.jms.tcp.TcpConnectionFactory;
import org.objectweb.petals.description.ContainerDescription;
import org.objectweb.petals.description.ServiceEndpointImpl;
import org.objectweb.petals.messaging.InOnlyImpl;
import org.objectweb.petals.messaging.InOptionalOutImpl;
import org.objectweb.petals.messaging.InOutImpl;
import org.objectweb.petals.messaging.MessageExchangeImpl;
import org.objectweb.petals.messaging.NormalizedMessageImpl;
import org.objectweb.petals.messaging.RobustInOnlyImpl;
import org.objectweb.petals.routing.Address;
import org.objectweb.petals.routing.Itinerary;
import org.objectweb.petals.transport.NormalizedMessageListener;
import org.objectweb.petals.transport.TransportException;
import org.objectweb.petals.transport.Transporter;
import org.objectweb.petals.util.ESBLogger;
import org.objectweb.petals.util.NamingHelper;
import org.objectweb.petals.util.StringHelper;


/**
 * @author Adrien LOUIS - EBM WebSourcing
 *
 */
public class TransporterImpl implements Transporter{
    
    public static final String ITINERARY_CURRENT   = "itinerary-current";
    public static final String ITINERARY_SERVICE_NAME   = "itinerary-service-name";
    public static final String ITINERARY_ENDPOINT_NAME   = "itinerary-endpoint-name";
    public static final String ITINERARY_SERVICE_CONTAINER_NAME   = "itinerary-service-container-name";
    public static final String ITINERARY_COMPONENT_NAME   = "itinerary-component-name";
    
    public static final String ID        = "id";
    public static final String STATUS    = "status";
    public static final String SERVICE   = "service";
    public static final String OPERATION = "operation";
    public static final String ENDPOINT  = "endpoint";
    public static final String PATTERN   = "pattern";
    public static final String REQUEST   = "request";
    
    private NormalizedMessageListener normalizedMessageListener;
    
    private ContainerDescription containerReference;
    
    private Queue queue;
    private QueueConnectionFactory qcf;
    private QueueSession qs;
    
    private int id;
    private int tcp;
    private String user;
    private String pwd;
    private String host;
    
    public TransporterImpl()
    {
        ESBLogger.logFine(this,"<init>");
        
    }
    
    public void setNormalizedMessageListener(NormalizedMessageListener normalizedMessageListener)
    {
        ESBLogger.logFine(this,"setNormalizedMessageListener("+normalizedMessageListener+")");
        
        this.normalizedMessageListener = normalizedMessageListener;
    }
    
    
    public void setContainerDescription(
            ContainerDescription serviceContainerReference) 
    {
        ESBLogger.logFine(this,"setContainerIdentification("+serviceContainerReference+")");
        
        this.containerReference= serviceContainerReference;
    }
    
    
    
    public void start() throws JBIException
    {
        ESBLogger.logFine(this,"start()");
        
        try{	
            Message msg;
            
            QueueConnection qc = qcf.createQueueConnection("root","root");
            qs = qc.createQueueSession(true, 0);
            QueueReceiver qrec = qs.createReceiver(queue);
            
            MessageListener aML = new MessageListener() {
                public void onMessage(Message aMessage) 
                {
                    processIncomingMessage(aMessage, qs);
                }
            };
            
            qrec.setMessageListener(aML);
            qc.start();
        }
        catch(JMSException ex)
        {
            ESBLogger.logSevere(this,"start()",ex);
            throw new JBIException(ex);
        }
    }
    
    /**
     * close the QueueSession,
     * but does not close the joram server
     */
    public void stop() throws JBIException
    {
        ESBLogger.logFine(this,"stop()");
        
        try {
            qs.commit();
            qs.close();
        } 
        catch (JMSException e) 
        {
            ESBLogger.logSevere(this,"stop()",e);
            throw new JBIException(e);
        }
    }
    
    public void init(HashMap ctx) throws JBIException 
    {
        ESBLogger.logFine(this,"init()");
        
        try 
        {
            initJoramVariables();
            JoramLifeCycle.getInstance().startServer();
            createUser("anonymous","anonymous");
            createQueue("queue-"+containerReference.getName());
        } 
        catch (Exception e) 
        {
            throw new JBIException(e);
        }
    }
    
    /**
     * init variables with joram.properties file
     * @throws Exception
     */
    private void initJoramVariables() throws Exception
    {
        ESBLogger.logFinest(this,"initJoramVariables()");
        
        // find server Id with "joram.properties" file
        URL url = ESBLogger.class.getResource("/joram.properties");
        Properties p = new Properties();
        p.load(url.openStream());
        id = Integer.parseInt(p.getProperty("id"));
        tcp = Integer.parseInt(p.getProperty("tcp"));
        user = p.getProperty("user");
        pwd = p.getProperty("pwd");
        host = p.getProperty("host");
    }
    
    
    /**
     * This method is called asynchronously when a message is received
     * @param msg
     * @param qs
     * TODO processIncomingMessage :manage the reception of unknown message type. PB between the calculation of the next recipient and notion of service provider/consumer 
     */
    private void processIncomingMessage(Message msg,QueueSession qs)
    {
        ESBLogger.logFinest(this,"processIncomingMessage("+msg+","+qs+")");
        
        StreamSource source = new StreamSource();
        
        if (msg instanceof TextMessage)
        {
            try
            {
                // create normalized message from jms content
                byte[] msgByte = ((TextMessage) msg).getText().getBytes();
                ByteArrayInputStream in = new ByteArrayInputStream(msgByte);
                source.setInputStream(in);
                NormalizedMessageImpl nm = new NormalizedMessageImpl();
                nm.setContent(source);
                // create normalized message properties from jms properties
                Enumeration e = msg.getPropertyNames();
                
                while(e.hasMoreElements())
                {
                    String propertyName = (String) e.nextElement();
                    Object propertyValue = msg.getObjectProperty(propertyName);
                    nm.setProperty(propertyName,propertyValue);
                }
                
                // create MessageExchange depending on pattern
                MessageExchangeImpl ms = null;
                String pattern = msg.getStringProperty(PATTERN);
                
                if(MessageExchangeImpl.IN_ONLY_PATTERN.toASCIIString().equals(pattern))
                {
                    ms = new InOnlyImpl();
                }
                else if(MessageExchangeImpl.IN_OPTIONAL_OUT_PATTERN.toASCIIString().equals(pattern))
                {
                    ms = new InOptionalOutImpl();
                }
                else if(MessageExchangeImpl.IN_OUT_PATTERN.toASCIIString().equals(pattern))
                {
                    ms = new InOutImpl();
                }
                else if(MessageExchangeImpl.ROBUST_IN_ONLY_PATTERN.toASCIIString().equals(pattern))
                {
                    ms = new RobustInOnlyImpl();
                }
                else
                {
                    ms = new MessageExchangeImpl();
                }
                
                // set message exchange information
                ms.setOperation(new QName(msg.getStringProperty(OPERATION)));
                
                // set message itinerary
                ms.setItinerary(decodeItinerary((TextMessage)(msg)));
                
                // set exchange properties (interface,service,endpoint)
                String endpoint = ms.getItinerary().getCurrent().getEndpoint();
                QName service = ms.getItinerary().getCurrent().getService();
                QName interfaceName = ms.getItinerary().getCurrent().getInterfaceName();
                ms.setEndpoint(new ServiceEndpointImpl(service, endpoint));
                ms.setService(service);
                ms.setInterfaceName(interfaceName);
                
                // set Id
                ms.setExchangeId(msg.getStringProperty(ID));
                
                // set status
                ms.setStatus(ExchangeStatus.valueOf(msg.getStringProperty(STATUS)));
                
                // set request typeS
                ms.setRequest(msg.getBooleanProperty(REQUEST));
                
                if(ms.isRequest())
                {
                    ms.setMessage(nm,MessageExchangeImpl.IN_MSG);
                    ms.setRole(MessageExchange.Role.PROVIDER);
                }
                else
                {
                    ms.setMessage(nm,MessageExchangeImpl.OUT_MSG);
                    ms.setRole(MessageExchange.Role.CONSUMER);
                }
                
                // poll message
                normalizedMessageListener.onMessage(ms);
            }
            catch(Exception e)
            {
                ESBLogger.logSevere(this,"processIncomingMessage",e);
            }
        }
        else
        {
            ESBLogger.logWarning(this,"processIncomingMessage : message is not a text message, do nothing");
            throw new RuntimeException("message is not a text message");
        }
    } 
    
    
    /**
     * Create and bind in JNDI a global connexionFactory and a global Queueconnexionfactory 
     * for all Joram servers if necessary.
     * Create and bind in JNDI a Queue for this server
     * @param queueName
     */
    private void createQueue(String queueName) throws Exception
    {
        ESBLogger.logFinest(this,"createQueue("+queueName+")");
        
        try
        {
            AdminModule.connect(host,tcp,user,pwd,3);
            javax.naming.Context ictx = new javax.naming.InitialContext();
            
            try 
            {
                ictx.unbind("qc");
            } 
            catch (RuntimeException e) 
            {
                ESBLogger.logSevere(this,"createQueue()",e);
            }    
            qcf=QueueTcpConnectionFactory.create();
            
            ictx.bind("qc",TcpConnectionFactory.create());
            ictx.unbind("qcf");
            ictx.bind("qcf",qcf);
            
            if(queue == null)
            {
                queue = Queue.create(queueName);
                queue.setFreeReading();
                queue.setFreeWriting();
                ictx.unbind(queueName);
                ictx.bind(queueName,queue);
            }
            ictx.close();
            AdminModule.disconnect();
            
        }
        catch(Exception ex)
        {
            ESBLogger.logSevere(this,"createQueue("+queueName+")",ex);
            throw ex;
        }
        
        
    }
    
    private void createUser(String u,String p) throws Exception
    {
        ESBLogger.logFinest(this,"createUser("+u+","+p+")");
        
        try
        {
            AdminModule.connect(host,tcp,user,pwd,3);
            User.create(u,p);
            AdminModule.disconnect();
        }
        catch(Exception ex)
        {
            ESBLogger.logSevere(this,"createUser",ex);
            throw ex;
        }
        
    }
    
    /**
     * @see javax.jbi.component.ComponentLifeCycle#shutDown()
     */
    public void shutDown() throws JBIException 
    {
        ESBLogger.logFine(this,"shutDown()");
            
        try
        {
            InitialContext ictx = new InitialContext();
            ictx.unbind("queue-"+containerReference.getName());
            JoramLifeCycle.getInstance().stopServer();
        }
        catch (NamingException ex)
        {
            ESBLogger.logSevere(this,"shutDown()",ex);
            throw new JBIException(ex);
        }
    }
    
    /**
     * @see petals.transport.TransportAgent#send(javax.jbi.messaging.MessageExchange, org.objectweb.petals.description.ContainerDescription)
     */
    public void send(MessageExchangeImpl messageExchange, Address address) throws TransportException{
        try
        {
            ESBLogger.logFine(this,"send("+messageExchange+","+address+")");
            /**
             if(address.getContainer().equals(containerReference.getName()))
             {
             // collocated destination
              
              ESBLogger.logFinest(this,"send("+messageExchange+","+address+") : collocated destination");
              
              normalizedMessageListener.onMessage(messageExchange);
              }
              else
              {
              */
            //distant destination
            ESBLogger.logFinest(this,"send("+messageExchange+","+address+") : distant destination");
            InitialContext ictx = new InitialContext();
            String remoteName = address.getContainer();
            
            // Find destination, create queue factory
            Queue remoteQueue = (Queue) ictx.lookup("queue-"+remoteName);
            QueueConnectionFactory qcf = (QueueConnectionFactory) ictx.lookup("qcf");
            ictx.close();
            QueueConnection qc = qcf.createQueueConnection("anonymous","anonymous");
            QueueSession qs = qc.createQueueSession(true, 0);
            QueueSender qsend = qs.createSender(remoteQueue);
            
            // create text message
            TextMessage msg = qs.createTextMessage();
            
            // set exchange properties
            String serviceName = NamingHelper.QNameToString(messageExchange.getService());
            String operationName = NamingHelper.QNameToString(messageExchange.getOperation());
            msg.setStringProperty(ID,messageExchange.getExchangeId());
            msg.setStringProperty(STATUS, messageExchange.getStatus().toString());
            msg.setStringProperty(SERVICE,serviceName==null?"":serviceName);
            msg.setStringProperty(OPERATION,operationName==null?"":operationName);
            msg.setStringProperty(PATTERN,messageExchange.getPattern().toASCIIString());
            msg.setStringProperty(ENDPOINT,address.getEndpoint());
            msg.setBooleanProperty(REQUEST, messageExchange.isRequest());
            encodeItinerary(msg,messageExchange.getItinerary());
            javax.jbi.messaging.NormalizedMessage nmToSend = null;
            
            if(messageExchange.isRequest())
            {
                nmToSend = messageExchange.getMessage(MessageExchangeImpl.IN_MSG);
            }
            else
            {
                nmToSend = messageExchange.getMessage(MessageExchangeImpl.OUT_MSG);
            }
            if(nmToSend == null)
            {
                throw new JBIException("normalized message to send is null");
            }
            for(Iterator it = nmToSend.getPropertyNames().iterator();it.hasNext();)
            {
                String pName = (String) it.next();
                msg.setObjectProperty(pName,nmToSend.getProperty(pName));
            }
            
            // set content
            StreamResult stream= new StreamResult(new ByteArrayOutputStream());
            Transformer transformer = TransformerFactory.newInstance().newTransformer();
            transformer.transform(nmToSend.getContent(),stream);
            stream.getOutputStream().close();
            msg.setText(stream.getOutputStream().toString());
            
            // send message
            qsend.send(msg);
            qs.commit();
            qc.close();
            //}
        }
        catch(Exception ex)
        {
            ESBLogger.logSevere(this,"send(). can not send message",ex);
            throw new TransportException(ex);
        }
        
    }
    
    /**
     * encode the itinerary in the textmessage
     * @param msg
     * @param itinerary
     * @throws JMSException
     */
    private void encodeItinerary(TextMessage msg, Itinerary itinerary) throws JMSException
    {
        ESBLogger.logFinest(this,"encodeItinerary("+msg+","+itinerary+")");
        
        msg.setIntProperty(ITINERARY_CURRENT, itinerary.indexOfCurrent());
        int index = 0;
        
        for(Iterator it = itinerary.itinerary();it.hasNext();index++)
        {
            Address address = (Address)it.next();
            String endpointName = address.getEndpoint();
            String serviceName = NamingHelper.QNameToString(address.getService());
            String containerName = address.getContainer();
            String componentName = address.getComponent();
            msg.setStringProperty(ITINERARY_SERVICE_NAME+index, serviceName==null?"":serviceName);
            msg.setStringProperty(ITINERARY_ENDPOINT_NAME+index, endpointName==null?"":endpointName);
            msg.setStringProperty(ITINERARY_SERVICE_CONTAINER_NAME+index, containerName==null?"":containerName);
            msg.setStringProperty(ITINERARY_COMPONENT_NAME+index,  componentName==null?"":componentName);
        }
    }
    
    /**
     * decode from JMS text message the encoded itinerary
     * @param msg
     * @return
     * @throws JMSException
     */
    private Itinerary decodeItinerary(TextMessage msg) throws JMSException
    {
        ESBLogger.logFinest(this,"decodeItinerary("+msg+")");
        
        String endpointName;
        String serviceName;
        String containerName;
        String componentName;
        String paramName;
        String endpointIndex;
        int current;
        
        Itinerary itinerary = new Itinerary();
        
        // find itinerary nodes
        boolean hasMore = true;
        int index = 0;
        
        while(hasMore)
        {
            containerName = msg.getStringProperty(ITINERARY_SERVICE_CONTAINER_NAME+index);
            
            if( containerName != null)
            {
                endpointName = msg.getStringProperty(ITINERARY_ENDPOINT_NAME+index);
                serviceName = msg.getStringProperty(ITINERARY_SERVICE_NAME+index);
                componentName = msg.getStringProperty(ITINERARY_COMPONENT_NAME+index);
                QName serviceQName = null;
                
                if(! StringHelper.isEmpty(serviceName))
                {
                    serviceQName = new QName(serviceName);
                }
                itinerary.addLast(new Address(null,serviceQName,endpointName,componentName,containerName));
                index++;
            }
            else{
                // no more nodes are found
                hasMore = false;
            }
            
        }
        
        // find currentNode index
        current = msg.getIntProperty(ITINERARY_CURRENT);
        itinerary.setCurrent(current);
        return itinerary;
    }
    
}