/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: DirectoryImplTest.java,v 1.1 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package com.ebmwebsourcing.jbi.systemservicesimpl;

import javax.jbi.JBIException;
import javax.xml.namespace.QName;

import org.objectweb.petals.description.ContainerDescription;
import org.objectweb.petals.routing.Address;
import org.objectweb.petals.util.NamingHelper;

import com.ebmwebsourcing.jbi.systemservicesimpl.DirectoryImpl;

import junit.framework.TestCase;

/**
 * @author mcarpentier
 */
public class DirectoryImplTest extends TestCase {
    
    DirectoryImpl dir ;
    /**
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception 
    {
        super.setUp();
    
        dir =new DirectoryImpl();;
        
        dir.init(null);
    }
    
    /**
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception 
    {
        super.tearDown();
    
        dir.stop();
        
        dir.shutDown();
    }
    
    public void testRegisterContainer() throws JBIException {
        
        // check container is registered
        ContainerDescription cd= new ContainerDescription("a","b",0);
        dir.registerContainer(cd);
        ContainerDescription cd2= dir.findContainerByName("a");
        assertEquals(cd,cd2);
        
        // check a second container with same name cannot be inserted
        boolean ok = true;
        try {
            dir.registerContainer(cd);
            ok = false;
        }catch (JBIException e) {}
        assertTrue(ok);
        
        // check a container with no name cannot be inserted
        ok = true;
        try {
            ContainerDescription cd3= new ContainerDescription(null,"b",0);
            dir.registerContainer(cd3);
            ok = false;
        }catch (JBIException e) {}
        assertTrue(ok);
                
        // check a container with no host cannot be inserted
        ok = true;
        try {
            ContainerDescription cd4= new ContainerDescription("c",null,0);
            dir.registerContainer(cd4);
            ok = false;
        } catch (JBIException e) {}
        assertTrue(ok);
        
        
        //  check 2 containers with same name but different host cannot be insterted
        ok = true;
        try {
            ContainerDescription cd5= new ContainerDescription("a","b2",0);
            dir.registerContainer(cd5);
            ok = false;
        } catch (JBIException e) {}
        assertTrue(ok);
        
	    // check an exception is thrown if null parameter
        ok = true;
        try {
            dir.registerContainer(null);
            ok = false;
        } catch (JBIException e) {}
        assertTrue(ok);
    }
    
    public void testUnregisterContainer() throws JBIException {
        ContainerDescription cd= new ContainerDescription("a","b",0);
        
        dir.registerContainer(cd);
        dir.unregisterContainer(cd);
        boolean ok = true;
        try {
            dir.unregisterContainer(cd);
            ok = false;
        } catch (JBIException e) {
            e.printStackTrace();
        }
        assertTrue(ok);
        
        ok = true;
        try {
            dir.unregisterContainer(null);
            ok = false;
        } catch (JBIException e) {
           e.printStackTrace();
        }
        assertTrue(ok);
        
        
        
    }
    
    public void testRegisterEndpoint() throws JBIException {
        ContainerDescription cd= new ContainerDescription("a","b",0);
        dir.registerContainer(cd);
        
        QName servicename = NamingHelper.StringToQName("test service");
        Address add;
        add = new Address(null,servicename,"endpoint","component",cd.getName());
        dir.registerEndpoint(add);
        
        Address add2 = dir.getAddressesForService(servicename)[0];
        
        assertEquals(add,add2);
        
        add2 = dir.getAddressForEndpoint("endpoint");
        assertEquals(add,add2);
        
        
        boolean ok = true;
        try {
            dir.registerEndpoint(add);
            ok = false;
        } catch (JBIException e) {
            e.printStackTrace();
        }
        assertTrue(ok);
        
        ok = true;
        try {
            add = new Address(null,null,"endpoint","component",cd.getName());
            dir.registerEndpoint(add);
            ok = false;
        } catch (JBIException e) {
            e.printStackTrace();
        }
        assertTrue(ok);
        
        ok = true;
        try {
            add = new Address(null,servicename,null,"component",cd.getName());
            dir.registerEndpoint(add);
            ok = false;
        } catch (JBIException e) {
            e.printStackTrace();
        }
        assertTrue(ok);
        
        
        ok = true;
        try {
            add = new Address(null,servicename,"endpoint",null,cd.getName());
            dir.registerEndpoint(add);
            ok = false;
        } catch (JBIException e) {
            e.printStackTrace();
        }
        assertTrue(ok);
        
        ok = true;
        try {
            add = new Address(null,servicename,"endpoint","component",null);
            dir.registerEndpoint(add);
            ok = false;
        } catch (JBIException e) {
             e.printStackTrace();
        }
        assertTrue(ok);
        
        
        ok = true;
        try {
            add = new Address(null,servicename,"endpoint","component","toto");
            dir.registerEndpoint(add);
            ok = false;
        } catch (JBIException e) {
            e.printStackTrace();
        }
        assertTrue(ok);
        
        
    }
    
    public void testUnregisterEndpoint() throws JBIException {
        ContainerDescription cd= new ContainerDescription("a","b",0);
        dir.registerContainer(cd);
        
        QName servicename = NamingHelper.StringToQName("test service");
        Address add;
        add = new Address(null,servicename,"endpoint","component",cd.getName());
        dir.registerEndpoint(add);
        
        dir.unregisterEndpoint(add);
        
        boolean ok = true;
        try {
            dir.unregisterEndpoint(add);
            ok = false;
        } catch (JBIException e) {
            e.printStackTrace();
        }
        assertTrue(ok);
        
        
    }
    
    public void testGetAddressesForInterface() throws JBIException{
        // not yet implemented
        dir.getAddressesForInterface(null);
    }
    
    public void testGetAddresses() throws JBIException {
        ContainerDescription cd= new ContainerDescription("a","b",0);
        dir.registerContainer(cd);
        ContainerDescription cd2= new ContainerDescription("c","d",0);
        dir.registerContainer(cd2);
        
        QName servicename = NamingHelper.StringToQName("test service");
        Address add = new Address(null,servicename,"endpoint","component",cd.getName());
        dir.registerEndpoint(add);
        Address add2 = new Address(null,servicename,"endpoint 2","component",cd.getName());
        dir.registerEndpoint(add2);
        
        boolean ok = true;
        try {
            Address add3 = new Address(null,servicename,"endpoint","component",cd2.getName());
            dir.registerEndpoint(add3);
            ok = false;
        } catch (JBIException e) {
            e.printStackTrace();
        }
        assertTrue(ok);
        
        
        assertEquals(dir.getAddresses()[0],add);
        
    }
    
    public void testGetAddressesForService() throws JBIException {
        ContainerDescription cd= new ContainerDescription("a","b",0);
        dir.registerContainer(cd);
        ContainerDescription cd2= new ContainerDescription("c","d",0);
        dir.registerContainer(cd2);
        
        QName servicename = NamingHelper.StringToQName("test service");
        Address add = new Address(null,servicename,"endpoint","component",cd.getName());
        dir.registerEndpoint(add);
        Address add2 = new Address(null,servicename,"endpoint 2","component",cd.getName());
        dir.registerEndpoint(add2);
        
        boolean ok = true;
        try {
            Address add3 = new Address(null,servicename,"endpoint","component",cd2.getName());
            dir.registerEndpoint(add3);
            ok = false;
        } catch (JBIException e) {
            e.printStackTrace();
        }
        assertTrue(ok);
        
        
        assertEquals(dir.getAddressesForService(servicename)[0],add);
        
        ok = true;
        try {
            Address[] add5 = dir.getAddressesForService(null);
            ok = false;
        } catch (JBIException e) {
            e.printStackTrace();
        }
        assertTrue(ok);
        
        Address[] add5 = dir.getAddressesForService(NamingHelper.StringToQName("test service inexistant"));
        assertEquals(add5.length,0);
        
    }
    
    public void testGetAddressForEndpoint() throws JBIException {
        
        ContainerDescription cd= new ContainerDescription("a","b",0);
        dir.registerContainer(cd);
        ContainerDescription cd2= new ContainerDescription("c","d",0);
        dir.registerContainer(cd2);
        
        QName servicename = NamingHelper.StringToQName("test service");
        Address add = new Address(null,servicename,"endpoint","component",cd.getName());
        dir.registerEndpoint(add);
        Address add2 = new Address(null,servicename,"endpoint 2","component",cd.getName());
        dir.registerEndpoint(add2);
        
        boolean ok = true;
        try {
            Address add3 = new Address(null,servicename,"endpoint","component",cd2.getName());
            dir.registerEndpoint(add3);
            
            ok = false;
        } catch (JBIException e) {
            e.printStackTrace();
        }
        assertTrue(ok);
        
        
        
        assertEquals(dir.getAddressForEndpoint("endpoint"),add);
        
        ok = true;
        try {
            Address add5 = dir.getAddressForEndpoint(null);
            ok = false;
        } catch (JBIException e) {
            e.printStackTrace();
        }
        assertTrue(ok);
        
        ok = true;
        try {
            Address add5 = dir.getAddressForEndpoint("inexistant");
            ok = false;
        } catch (JBIException e) {
            e.printStackTrace();
        }
        assertTrue(ok);
        
        
        
        
        
        
    }
    
    public void testFindContainerByName() throws JBIException {
        ContainerDescription cd = dir.findContainerByName("aaaaa");
        assertNull(cd);
        
        boolean ok = true;
        try {
            cd = dir.findContainerByName(null);
            ok = false;
        } catch (JBIException e) {
            e.printStackTrace();
        }
        assertTrue(ok);
        
        
        
        
        
        
    }
    
    public void testInitContexts() {
    }
    
    public void testGetNamingContext() {
    }
    
}
