/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: Client.java,v 1.1 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package com.ebmwebsourcing.jbi.sample.service.client;

import javax.jbi.JBIException;

import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOut;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;

import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.swing.JFrame;

import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JTextArea;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;

import org.objectweb.petals.util.ESBLogger;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;

import com.ebmwebsourcing.jbi.sample.service.util.SourceHelper;

/**
 * @author Adrien LOUIS - EBM WebSourcing
  */
public class Client extends JFrame implements Component, ComponentLifeCycle{
    
    private javax.swing.JPanel jContentPane = null;
    
    private JPanel jPanel = null;
    private JPanel jPanel1 = null;
    private JTextField jTextField = null;
    private JButton jButton = null;
    private JTextArea jTextArea = null;
    
     
    /**
     * This is the default constructor
     */
    public Client() {
        super();
        initialize();
    }
    /**
     * This method initializes this
     * 
     * @return void
     */
    private void initialize() {
        this.setSize(569, 331);
        this.setContentPane(getJContentPane());
        this.setTitle("ESB Client test");
    }
    /**
     * This method initializes jContentPane
     * 
     * @return javax.swing.JPanel
     */
    private javax.swing.JPanel getJContentPane() {
        if(jContentPane == null) {
            jContentPane = new javax.swing.JPanel();
            jContentPane.setLayout(new java.awt.BorderLayout());
            jContentPane.add(getJPanel(), java.awt.BorderLayout.NORTH);
            jContentPane.add(getJPanel1(), java.awt.BorderLayout.CENTER);
        }
        return jContentPane;
    }
    /**
     * This method initializes jPanel	
     * 	
     * @return javax.swing.JPanel	
     */    
    private JPanel getJPanel() {
        if (jPanel == null) {
            jPanel = new JPanel();
            jPanel.add(getJTextField(), null);
            jPanel.add(getJButton(), null);
        }
        return jPanel;
    }
    /**
     * This method initializes jPanel1	
     * 	
     * @return javax.swing.JPanel	
     */    
    private JPanel getJPanel1() {
        if (jPanel1 == null) {
            jPanel1 = new JPanel();
            jPanel1.add(getJTextArea(), null);
        }
        return jPanel1;
    }
    /**
     * This method initializes jTextField	
     * 	
     * @return javax.swing.JTextField	
     */    
    private JTextField getJTextField() {
        if (jTextField == null) {
            jTextField = new JTextField();
            jTextField.setPreferredSize(new java.awt.Dimension(300,20));
        }
        return jTextField;
    }
    /**
     * This method initializes jButton	
     * 	
     * @return javax.swing.JButton	
     */    
    private JButton getJButton() {
        if (jButton == null) {
            jButton = new JButton();
            jButton.setText("send to HelloService");
            jButton.addActionListener(new java.awt.event.ActionListener() { 
                public void actionPerformed(java.awt.event.ActionEvent e) {    
                    sendMessage();
                }
            });
        }
        return jButton;
    }
    /**
     * This method initializes jTextArea	
     * 	
     * @return javax.swing.JTextArea	
     */    
    private JTextArea getJTextArea() {
        if (jTextArea == null) {
            jTextArea = new JTextArea();
            jTextArea.setColumns(1);
            jTextArea.setComponentOrientation(java.awt.ComponentOrientation.LEFT_TO_RIGHT);
            jTextArea.setPreferredSize(new java.awt.Dimension(300,60));
            jTextArea.setRows(15);
            jTextArea.setText("received messages are printed here :");
        }
        return jTextArea;
    }
    
    
    ////////////////////////////////
    ////////////////////////////////
    // Component Impl
    
    
    private ComponentContext context;
    
    private ClientListener listener;
    
    private void sendMessage()
    {
        try {
            String content = getJTextField().getText();
            
            Source source = SourceHelper.createContentSource(content);
            
            InOut msg = context.getDeliveryChannel().createExchangeFactory().createInOutExchange();
            
            NormalizedMessage nm = msg.createMessage();
            
            nm.setContent(source);
            
            msg.setService(new QName("HelloService"));
            
            msg.setOperation(new QName("hello"));
            
            msg.setInMessage(nm);
            
            context.getDeliveryChannel().send(msg);
            
        } catch (Exception e) {
            ESBLogger.logSevere("Client sendMessage",e);
        }
    }
    
    public void init(ComponentContext context) throws JBIException
    {
        ESBLogger.logFine(this,"init()");
        
        try {
            this.context = context;
        } 
        catch (Exception e) 
        {
            ESBLogger.logSevere(this,"init()",e);
            throw new JBIException(e);
        }
    }
    
    public void start() throws JBIException
    {
        ESBLogger.logFine(this,"start()");
        
        this.setVisible(true);
        
        listener = new ClientListener(this,context.getDeliveryChannel());
        
        //endpoint = context.activateEndpoint(null, context.getComponentName());
        
        listener.start();
    }
    
    public void stop() throws JBIException
    {
        ESBLogger.logFine(this,"stop()");
        
        listener.running = false;
    }
    
    public ObjectName getExtensionMBeanName() {
        return null;
    }
    
    public void shutDown() throws JBIException {
        ESBLogger.logFine(this,"shutDown()");
    } 
    
    class ClientListener extends Thread{
        
        DeliveryChannel channel;
        
        Client client;
        
        boolean running;
        
        ClientListener(Client client,DeliveryChannel channel)
        {
            this.channel = channel;
            
            this.client = client;
        }
        
        public void run()
        {
            ESBLogger.logFine(this,"run()");
            
            running = true;
            
            ESBLogger.logFinest(this,"run() listening...");
            
            MessageExchange msgEx;
            
            while(running)
            {
                try 
                {
                    msgEx = channel.accept();
                    
                    if(msgEx instanceof InOut)
                    {
                        processInOut((InOut)msgEx);
                    }
                    else
                    {
                        ESBLogger.logWarning(this,"run() :message type not recognized : "+msgEx.getPattern());
                    }
                }
                catch (MessagingException e) {
                    ESBLogger.logSevere(this,"run()",e);
                }
            }
            
            ESBLogger.logFinest(this,"run() stop listening.");
        }
        
        private void processInOut(InOut msg)
        {
            ESBLogger.logFine(this,"processInOut("+msg+")");
            
            try
            {
                try 
                {
                    String content = SourceHelper.createStringContent(msg.getOutMessage().getContent());
                    
                    msg.setStatus(ExchangeStatus.DONE);
                    
                    client.getJTextArea().append("\n"+content);
                } 
                catch (Exception e) 
                {
                    ESBLogger.logSevere(this,"processInOut()",e);
                    
                    msg.setStatus(ExchangeStatus.ERROR);
                    
                    msg.setError(e);
                }
            }
            catch (MessagingException e) 
            {
                ESBLogger.logSevere(this,"processInOut()",e);
            } 
            
        }
    }
    
    
    public ComponentLifeCycle getLifeCycle() {
        return this;
    }
    
    
    public ServiceUnitManager getServiceUnitManager() {
        return null;
    }
    
    
    public Document getServiceDescription(ServiceEndpoint ref)
    {
        return null;
    }
    public boolean isExchangeWithConsumerOkay(ServiceEndpoint arg0,
            MessageExchange arg1)
    {
        throw new NoSuchMethodError();
    }
    public boolean isExchangeWithProviderOkay(ServiceEndpoint arg0,
            MessageExchange arg1)
    {
        throw new NoSuchMethodError();
    }
    public ServiceEndpoint resolveEndpointReference(DocumentFragment arg0)
    {
        throw new NoSuchMethodError();
    }
}  
