/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: HelloService.java,v 1.1 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package com.ebmwebsourcing.jbi.sample.service.hello;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;


import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import org.objectweb.petals.util.ESBLogger;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;

import com.ebmwebsourcing.jbi.sample.service.util.SourceHelper;


/**
 * @author Adrien LOUIS - EBM WebSourcing
 *
 */
public class HelloService implements Component, ComponentLifeCycle{
    
    private ComponentContext context;
    
    private HelloServiceListener listener;
    
    private static final String ENDPOINT = "EndpointForHelloService";
    private static final String SERVICE = "HelloService";
    
    private ServiceEndpoint endpointReference;
    
    public void init(ComponentContext context) throws JBIException
    {
        ESBLogger.logFine(this,"init()");
        ESBLogger.logInfo(this, "Initialize the Hello service");
        
        try {
            this.context = context;
            
            } 
        catch (Exception e) 
        {
            ESBLogger.logSevere(this,"init()",e);
            throw new JBIException(e);
        }
        
        
    }
    
    public void start() throws JBIException
    {
        ESBLogger.logFine(this,"start()");
        ESBLogger.logInfo(this, "Start the Hello service");
        
        listener = new HelloServiceListener(context.getDeliveryChannel());
             
             listener.start();
             
              endpointReference = context.activateEndpoint(new QName(SERVICE), "Endpoint on "+context.getComponentName());        
    }
    
    public void stop() throws JBIException
    {
        ESBLogger.logFine(this,"stop()");
        ESBLogger.logInfo(this, "Stop the Hello service");
        
      context.deactivateEndpoint(endpointReference);

  listener.running = false;
 		
        
    }
    
    public ObjectName getExtensionMBeanName() {
        return null;
    }
    
    public void shutDown() throws JBIException {
        ESBLogger.logFine(this,"shutDown()");
        ESBLogger.logInfo(this, "Shutdown the Hello service");
        try
        {
            Thread.sleep(1000);
        }
        catch (InterruptedException e)
        {
            throw new JBIException(e);
        }
    } 
    
    class HelloServiceListener extends Thread{
        
        DeliveryChannel channel;
        
        boolean running;
        
        HelloServiceListener(DeliveryChannel channel)
        {
            this.channel = channel;
        }
        
        public void run()
        {
            ESBLogger.logFine(this,"run()");
            
            running = true;
            
            ESBLogger.logFinest(this,"run() listening...");
            
            MessageExchange msgEx;
            
            while(running)
            {
                try 
                {
                    msgEx = channel.accept();
                    
                    if(msgEx instanceof InOnly)
                    {
                        processInOnly((InOnly)msgEx);
                    }
                    else if(msgEx instanceof InOut)
                    {
                        processInOut((InOut)msgEx);
                    }
                }
                catch (MessagingException e) {
                    ESBLogger.logSevere(this,"run()",e);
                }
            }
            
            ESBLogger.logFinest(this,"run() stop listening.");
        }
        
        private void processInOnly(InOnly msg)
        {
            ESBLogger.logFinest(this,"processInOnly("+msg+")");
            
            try
            {
                if(new QName("hello").equals(msg.getOperation()))
                {
                    String nom = SourceHelper.createStringContent(msg.getInMessage().getContent());
                    
                    System.out.println("HelloServiceListener: '"+nom+"' said hello with InOnlyMessage");
                    
                    if(nom == null || nom.trim().length() == 0)
                        throw new Exception("name is null");
                
                }
                else
                    throw new Exception("only 'hello' operation is recognized");
            
            }
            catch (Exception e) 
            {
                ESBLogger.logSevere(this,"processInOnly()",e);
            } 
            
        }
        
        /**
         * Process message.
         * Only "hello(string name)" operation is recognized
         * if no name is specified or other operation is specified,
         * throw an exception
         * TODO processInOut : if a fault occurs, send it somewhere
         * @param msg
         */
        private void processInOut(InOut msg)
        {
            ESBLogger.logFinest(this,"processInOut("+msg+")");
            try 
            {
                if(new QName("hello").equals(msg.getOperation()))
                {
                    String nom = SourceHelper.createStringContent(msg.getInMessage().getContent());
                    
                    System.out.println("Hello Service : '"+nom+"' said hello with InOutMessage");
                    
                    if(nom == null || nom.trim().length() == 0)
                        throw new Exception("name is null");
                    
                    NormalizedMessage nm = msg.createMessage();
                    
                    nm.setContent(SourceHelper.createContentSource("hello "+nom));
                    
                    msg.setOutMessage(nm);
                    
                    context.getDeliveryChannel().send(msg);
                }
                else
                    throw new Exception("only 'hello' operation is recognized");
            } 
            catch (Exception e) 
            {
                ESBLogger.logSevere(this,"processInOut()",e);
                
                try {
                    msg.setStatus(ExchangeStatus.ERROR);
                    
                    Fault f  = msg.createFault();
                    
                    f.setContent(SourceHelper.createSource("fault hello "+e.getMessage()));
                    
                    msg.setFault(f);
                    
                } catch (MessagingException e1) {
                    ESBLogger.logSevere(this,"processInOut()",e);
                }
                
                msg.setError(e);
            }
            
        }
        
    }
    
   
    
    public ComponentLifeCycle getLifeCycle() {
        return this;
    }
    
    
    public ServiceUnitManager getServiceUnitManager() {
        return null;
    }
    
    public Document getServiceDescription(ServiceEndpoint ref)
    {
        return null;
    }
    
    public boolean isExchangeWithConsumerOkay(ServiceEndpoint arg0,
            MessageExchange arg1)
    {
        throw new NoSuchMethodError();
    }
    public boolean isExchangeWithProviderOkay(ServiceEndpoint arg0,
            MessageExchange arg1)
    {
        throw new NoSuchMethodError();
    }
    public ServiceEndpoint resolveEndpointReference(DocumentFragment arg0)
    {
        throw new NoSuchMethodError();
    }
   
}
