/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005  EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: TransformerEngine.java,v 1.1 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------  
 */

package com.ebmwebsourcing.jbi.sample.service.transformer;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import org.objectweb.petals.util.ESBLogger;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;

import com.ebmwebsourcing.jbi.sample.service.util.SourceHelper;

/**
 * @author Adrien LOUIS - EBM WebSourcing
*/
public class TransformerEngine implements Component, ComponentLifeCycle
{

    private ComponentContext context;
    
    private TransformerEngineListener listener;
    
    private static final String ENDPOINT = "EndpointForTransformerEngine";
    private static final String SERVICE = "Transformer";
    
    private ServiceEndpoint endpointReference;
    
    public void init(ComponentContext context) throws JBIException
    {
        ESBLogger.logFine(this,"init()");
        
        ESBLogger.logInfo(this, "Initialize the transformer service");
        try {
            this.context = context;
            
            } 
        catch (Exception e) 
        {
            ESBLogger.logSevere(this,"init()",e);
            throw new JBIException(e);
        }
        
        
    }
    
    public void start() throws JBIException
    {
        ESBLogger.logFine(this,"start()");

        ESBLogger.logInfo(this, "Start the transformer service");
        
               listener = new TransformerEngineListener(context.getDeliveryChannel());
             
             listener.start();
             
              endpointReference = context.activateEndpoint(new QName(SERVICE), "Endpoint on "+context.getComponentName());
       }
    
    public void stop() throws JBIException
    {
        ESBLogger.logFine(this,"stop()");
  
        ESBLogger.logInfo(this, "Stop the transformer service");

        context.deactivateEndpoint(endpointReference);

        listener.running = false;
     }
    
    public ObjectName getExtensionMBeanName() {
        return null;
    }
    
    public void shutDown() throws JBIException {
        ESBLogger.logFine(this,"shutDown()");
        
        ESBLogger.logInfo(this, "Shutdown the transformer service");
        try
        {
            Thread.sleep(1000);
        }
        catch (InterruptedException e)
        {
            throw new JBIException(e);
        }
        
    } 
    
    class TransformerEngineListener extends Thread{
        
        DeliveryChannel channel;
        
        boolean running;
        
        public TransformerEngineListener(DeliveryChannel channel)
        {
            this.channel = channel;
        }
        
        public void run()
        {
            ESBLogger.logFine(this,"run()");
            
            running = true;
            
            ESBLogger.logFinest(this,"run() listening...");
            
            MessageExchange msgEx;
            
            while(running)
            {
                try 
                {
                    msgEx = channel.accept();
                    
                    if(msgEx instanceof MessageExchange)
                    {
                        process((MessageExchange)msgEx);
                    }
                    
                }
                catch (MessagingException e) {
                    ESBLogger.logSevere(this,"run()",e);
                }
            }
            
            ESBLogger.logFinest(this,"run() stop listening.");
        }
        
        /**
         * TODO process: if a fault occurs, send it somewhere
         * @param msg
         */
        private void process(MessageExchange msg)
        {
            ESBLogger.logFine(this,"process("+msg+")");
            
            try 
            {
                    String nom = SourceHelper.createStringContent(msg.getMessage("in").getContent());
                    
                    if(nom == null || nom.trim().length() == 0)
                        throw new Exception("name is null");
                    
                    System.out.println("Transform Engine : '"+nom+"' to upper case :"+nom.toUpperCase());
                    
                    msg.getMessage("in").setContent(SourceHelper.createContentSource(nom.toUpperCase()));
                    
                    context.getDeliveryChannel().send(msg);
            } 
            catch (Exception e) 
            {
                ESBLogger.logSevere(this,"process()",e);
                
                try {
                    msg.setStatus(ExchangeStatus.ERROR);
                    
                    Fault f  = msg.createFault();
                    
                    f.setContent(SourceHelper.createSource("fault transformerEngine "+e.getMessage()));
                    
                    msg.setFault(f);
                    
                    
                } catch (MessagingException e1) {
                    ESBLogger.logSevere(this,"process()",e);
                }
                
                msg.setError(e);
            }
            
        }
        
    }
    
   
    
    public ComponentLifeCycle getLifeCycle() {
        return this;
    }
    
    
    public ServiceUnitManager getServiceUnitManager() {
        return null;
    }
    
    
    
    public Document getServiceDescription(ServiceEndpoint ref)
    {
        return null;
    }
    
    public boolean isExchangeWithConsumerOkay(ServiceEndpoint arg0,
            MessageExchange arg1)
    {
        throw new NoSuchMethodError();
    }
    public boolean isExchangeWithProviderOkay(ServiceEndpoint arg0,
            MessageExchange arg1)
    {
        throw new NoSuchMethodError();
    }
    public ServiceEndpoint resolveEndpointReference(DocumentFragment arg0)
    {
        throw new NoSuchMethodError();
    }
   

}
